# MonArch - Groundwork Monitor Architect
# MonarchLoad.pm
#
###############################################################################
# Release 0.97a
# 30-Sept-2005
###############################################################################
#

use DBI;
use XML::LibXML;
use strict;
use MonarchConf;

package Load;

my $debug = 0;
my $is_portal = 0; # Set this value to 1 when running in the Portal
my ($dbhost, $database,$user,$passwd) = undef;
if ($is_portal) {
	open(FILE, "< /usr/local/groundwork/config/db.properties");
	while (my $line = <FILE>) {
		if ($line =~ /\s*monarch\.dbhost\s*=\s*(\S+)/) { $dbhost = $1 }
		if ($line =~ /\s*monarch\.database\s*=\s*(\S+)/) { $database = $1 }
		if ($line =~ /\s*monarch\.username\s*=\s*(\S+)/) { $user = $1 }
		if ($line =~ /\s*monarch\.password\s*=\s*(\S+)/) { $passwd = $1 }
	}
	close(FILE);
} else {
	($dbhost, $database,$user,$passwd) = Conf->get_dbauth();
}
my $dsn = "DBI:mysql:$database:$dbhost";
my $dbh = DBI->connect($dsn, $user, $passwd, {'RaiseError' => 1});

my @messages = ();
my $xml_begin = "<?xml version=\"1.0\" ?>\n<data>\n";
my $xml_end = "</data>";
my $dt = 0;

sub parse_xml($) {
	my $data = shift;
	my %properties = ();
	if ($data) {
		eval {
			my $parser = XML::LibXML->new();
			my $doc = $parser->parse_string($data);
			my @nodes = $doc->findnodes( "//prop" );
			foreach my $node (@nodes) {
				if ($node->hasAttributes()) {
					my $property = $node->getAttribute('name');
					my $value = $node->textContent;
					$value =~ s/\s+$|\n//g;
					if ($property =~ /command$/) {
						if ($value =~ /!/) { 
							$properties{'command_line'} = $value;
							my @cmd = split(/!/, $value);	
							$properties{'check_command'} = $cmd[0];
						} else {
							$properties{'check_command'} = $value;
						}
					} elsif ($property =~ /last_notification$/) {
						my $value = $node->textContent;
						$value =~ s/\s+$|\n//g;
						if ($value == 0) {
							$properties{$property} = '-zero-';
						} else {					
							$properties{$property} = $value;
						}
					} else {
						$properties{$property} = $value;
					}
				}
			}
		};
	}
	if ($@) {
		update_status('error',$@);
		if ($debug) { print "\nError parsing: $@ \n$data\n"; my $x = <STDIN> }
	} else {
		return %properties;
	}
}

sub fetch_list_where(@) {
	my $table = $_[0];
	my $list = $_[1];
	my $where = $_[2];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']]%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				$where_clause .= " $name = '$where{$name}' and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "select $list from $table where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @elements = ();
	my @values = ();
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub fetch_list_hash_array(@) {
	my $table = $_[0];
	my $where = $_[1];
	my %where = %{$where};
	my %elements = ();
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH|NONE/) {
			$where_clause .= " $name = '$where{$name}' and"; 
		}
	}
	$where_clause =~ s/ and$//;
	if ($where_clause) { $where_clause = " where $where_clause" }
	my $sqlstmt = "select * from $table$where_clause";
	my @values = undef;
	my $sth = $dbh->prepare($sqlstmt);
	if ($sth->execute) {
		while(@values = $sth->fetchrow_array()) {
			$elements{$values[0]} = [ @values ];
		}
		return %elements;
	} else {
		$elements{'error'} = "Error: $@";
		return %elements;
	}
	$sth->finish;
}




sub fetch_one(@) {
	my $table = $_[0];
	my $name = $_[1];
	my $value = $_[2];
	my $values = undef;
	my %properties = ();
	$value = $dbh->quote($value);
	my $sqlstmt = "select * from $table where $name = $value";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	$values = $sth->fetchrow_hashref();
	foreach my $key (keys %{$values}) {
		if ($key eq 'data') {
			my %data = parse_xml($values->{$key});
			foreach my $k (keys %data) {
				$properties{$k} = $data{$k};
			}
		} else {
			$properties{$key} = $values->{$key};
		}
	}
	$sth->finish;
	return %properties;
}

sub fetch_one_where(@) {
	my $table = $_[0];
	my $where = $_[1];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				$where_clause .= " $name = '$where{$name}' and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "select * from $table where $where_clause";
	my $sth = undef;
	eval{
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute();
	};
	if ($@) {
		update_status('error',$@);
		if ($debug) { print "\nError: $@\n sql: $sqlstmt\n" }
	} else {
		my $values = $sth->fetchrow_hashref();
		my %properties = ();
		foreach my $key (keys %{$values}) {
			if ($key eq 'data') {
				my %data = parse_xml($values->{$key});
				foreach my $k (keys %data) {
					$properties{$k} = $data{$k};
				}
			} else {
				$properties{$key} = $values->{$key};
			}
		}
		$sth->finish;
		return %properties;
	}
}

sub fetch_list(@) {
	my $table = $_[0];
	my $list = $_[1];
	my @values = ();
	my @elements = ();
	my $sqlstmt = "select $list from $table";
	my $sth = $dbh->prepare ($sqlstmt);
	eval{
		$sth->execute;
		while(@values = $sth->fetchrow_array()) {
			push @elements, $values[0];
		}
		$sth->finish;
	};
	if ($@) {
		update_status('error',$@);
		if ($debug) { print "\nError: $@\n sql: $sqlstmt\n" }
	} else {
		return @elements;
	}
}

sub fetch_distinct_list(@) {
	my $table = $_[0];
	my $list = $_[1];
	my @values = ();
	my @elements = ();
	my $sqlstmt = "select distinct $list from $table";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub insert_obj(@) {
	my $table = $_[0];
	my $values =  $_[1];
	my @values = @{$values};
	my $valstr = undef;
	foreach my $val (@values) {
		$val = $dbh->quote($val);
		$valstr .= "$val,";
	}
	chop $valstr;
	my $sqlstmt = "insert into $table values($valstr)";
	my $sth = undef;
	eval{
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute();
	};	
	$sth->finish;
	if ($@) {
		my $type = 'error';
		my $err = $@;
		if ($table eq 'hostgroup_host' && $err =~ /Duplicate/i) { 
			$type = 'note';
			my %t = fetch_one('hostgroups','hostgroup_id',$values[0]);
			my %n = fetch_one('hosts','host_id',$values[1]);
			$err = "Duplicate name $n{'name'} in member list for hostgroup $t{'name'} ignored.";
		} elsif ($table eq 'contactgroup_contact' && $err =~ /Duplicate/i) {
			$type = 'note';
			my %t = fetch_one('contactgroups','contactgroup_id',$values[0]);
			my %n = fetch_one('contacts','contact_id',$values[1]);
			$err = "Duplicate name $n{'name'} in member list for contact group $t{'name'} ignored.";
		} elsif ($err =~ /Duplicate/i) {
			$type = 'note';
			$err = "Duplicate $table $values[0] will be ignored.";
		}
		update_status($type,$err);
		if ($debug) { print "\nError: $err\n sql: $sqlstmt\n\n----press return to continue:"; my $x = <STDIN> }
	} 
}

sub insert_obj_id(@) {
	my $table = $_[0];
	my $values =  $_[1];
	my $id =  $_[2];
	my @values = @{$values};
	my $valstr = undef;
	foreach my $val (@values) {
		$val = $dbh->quote($val);
		$valstr .= "$val,";
	}
	chop $valstr;
	my $sqlstmt = "insert into $table values($valstr)";
	eval{
		my $sth = $dbh->prepare ($sqlstmt);
		$sth->execute();
		$sth->finish;
	};
	if ($@) {
		update_status('error',$@);
		if ($debug) { print "\nError: $@\n sql: $sqlstmt\n\n----press return to continue:"; my $x = <STDIN> }
	} else {
		my $return_id = undef;
		eval{
			$sqlstmt = "select max($id) as lastrow from $table";
			$return_id = $dbh->selectrow_array($sqlstmt);
		};
		if ($@) {
			update_status('error',$@);
			if ($debug) { print "\nError: $@\n sql: $sqlstmt\n\n----press return to continue:"; my $x = <STDIN> }
		} else {
			return $return_id;
		}
	}
}

sub delete_all(@) {
	my $table = $_[0];
	my $name = $_[1];
	my $obj = $_[2];
	my $sqlstmt = "delete from $table where $name = '$obj'";
	my $sth = $dbh->prepare ($sqlstmt);
	unless ($sth->execute) {
		update_status('error',$@);
		if ($debug) { print "\nError: $@\n sql: $sqlstmt\n\n----press return to continue:"; my $x = <STDIN> }
	}
	$sth->finish;
}

sub delete_one_where(@) {
	my $table = $_[0];
	my $where = $_[1];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']]%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				$where_clause .= " $name = '$where{$name}' and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "delete from $table where $where_clause";
	my $sth = undef;
	eval{
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute();
	};
	$sth->finish;
	if ($@) {
		update_status('error',$@);
		if ($debug) { print "\nError: $@\n sql: $sqlstmt\n" }
	} 
}

sub update_obj(@) {
	my $table = $_[0];
	my $name = $_[1];
	my $obj = $_[2];
	my $values =  $_[3];
	my %values = %{$values};
	my $valstr = undef;
	$obj = $dbh->quote($obj);
	foreach my $key (keys %values) {
		unless ($key =~ /HASH/) {
			my $val = $dbh->quote($values{$key});
			$valstr .= "$key = $val,";
		}
	}
	chop $valstr;
	my $sqlstmt = "update $table set $valstr where $name = $obj";
	my $sth = $dbh->prepare ($sqlstmt);
	unless ($sth->execute) {
		update_status('error',$@);
		if ($debug) { print "\nError: $@\n sql: $sqlstmt\n\n----press return to continue:"; my $x = <STDIN> }
	}
	$sth->finish;
}

sub update_obj_where(@) {
	my $table = $_[0];
	my $values =  $_[1];
	my $where = $_[2];
	my %values = %{$values};
	my %where = %{$where};
	my $valstr = undef;
	foreach my $key (keys %values) {
		unless ($key =~ /HASH/) {
			my $val = $dbh->quote($values{$key});
			$valstr .= "$key = $val,";
		}
	}
	chop $valstr;
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			$where_clause .= " $name = '$where{$name}' and"; 
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "update $table set $valstr where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	unless ($sth->execute) {
		update_status('error',$@);
		if ($debug) { print "\nError: $@\n sql: $sqlstmt\n\n----press return to continue:"; my $x = <STDIN> }
	}
	$sth->finish;
}

sub datetime() {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	$year += 1900;
	$mon++;
	if ($mon =~ /^\d{1}$/) { $mon = "0".$mon }
	if ($mday =~ /^\d{1}$/) { $mday = "0".$mday }
	if ($hour =~ /^\d{1}$/) { $hour = "0".$hour }
	if ($min =~ /^\d{1}$/) { $min = "0".$min }
	if ($sec =~ /^\d{1}$/) { $sec = "0".$sec }
	return "$year-$mon-$mday $hour:$min:$sec";
}

sub count(@) {
	my $sqlstmt = "select count(*) from $_[0]";
	my $count = $dbh->selectrow_array($sqlstmt);
	return $count;
}

sub update_status(@) {
	my $type = $_[0];
	my $message = $_[1];
	if ($debug) { print "\n$type::$message\n" }
	my @msg = ($type,$message);
	push @messages, [ @msg ];
}

sub truncate_table(@) {
	my $table = $_[0];
	my $sqlstmt = "truncate table $table";
	my $sth = undef;
	eval{
		$sth = $dbh->prepare($sqlstmt);
		$sth->execute();
	};
	if ($@) {
		update_status('error',$@);
		if ($debug) { print "\nError: $@\n sql: $sqlstmt\n" }
	}
}

sub truncate_tables() {
	my $tables = qq(commands
contact_command
contact_templates
contactgroup_assign
contactgroup_contact
contactgroups
contacts
escalation_templates
escalation_tree_template
escalation_trees
extended_host_info_templates
extended_info_coords
extended_service_info_templates
host_dependencies
host_parent
host_templates
hostgroup_host
hostgroups
hosts
host_overrides
service_dependency
service_dependency_templates
service_templates
servicegroup_service
servicegroups
services
profile_parent
profile_hostgroup
profiles_host
profiles_service
service_names
service_overrides
time_periods
tree_template_contactgroup);
	my @tables = split(/\n/, $tables);
	foreach my $table (@tables) {
		truncate_table($table);
	}
	return @messages;
}

#
###########################################################################################################################
# Stage
#

sub stage_load(@) {
	my $nagios_home = $_[1];
	my $index = 0;
	my @results = ();
	my %template = ();
	my %host_template_reassign = ();
	my %parents = ();
	my %parent_host = ();
	my %dependency = ();
	my %service_dep = ();
	my %file_type = ();
	my %file_object = ('commands' => 'checkcommands.cfg',
		'miscommands' => 'miscommands.cfg',
		'contact_templates' => 'contact_templates.cfg',
		'contactgroups' => 'contactgroups.cfg',
		'contacts' => 'contacts.cfg',
		'escalation_templates' => 'escalation_templates.cfg',
		'escalations' => 'escalations.cfg',
		'extended_host_info_templates' => 'extended_host_info_templates.cfg',
		'extended_host_info' => 'extended_host_info.cfg',
		'extended_service_info_templates' => 'extended_service_info_templates.cfg',
		'extended_service_info' => 'extended_service_info.cfg',
		'host_dependencies' => 'host_dependencies.cfg',
		'host_templates' => 'host_templates.cfg',
		'hostgroups' => 'hostgroups.cfg',
		'service_dependency' => 'service_dependency.cfg',
		'service_dependency_templates' => 'service_dependency_templates.cfg',
		'service_templates' => 'service_templates.cfg',
		'servicegroups' => 'servicegroups.cfg',
		'time_periods' => 'time_periods.cfg',
		'services' => 'services.cfg',
		'hosts' => 'hosts.cfg');
	my $sqlstmt = "truncate table stage_status";
	my $sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('error', "Error: $sqlstmt $@") if $@;
	my $dt = datetime();
	update_status('start',$dt);
	my @stage_tables = ('stage_escalations','stage_host_hostgroups','stage_host_services','stage_hosts','stage_other');
	foreach (@stage_tables) { truncate_table($_) }
	my @file_tables = ('files','file_host','file_service');
	foreach (@file_tables) { truncate_table($_) }
	update_status('info', "Reading: $nagios_home/nagios.cfg");
	$sqlstmt = "delete from setup where type = 'file'";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('error', "Error: $sqlstmt $@") if $@;

	my %file_paths = ();
	my @files = ();
	my @dirs = ();
	my $resource_file = undef;
	open(FILE, "< $nagios_home/nagios.cfg") or update_status('error',"error: cannot open $nagios_home/nagios.cfg to read $!");
	while (my $line = <FILE>) {
		if ($line =~ /^cfg_file=(\S+)$/) { push @files, $1 }
		if ($line =~ /^cfg_dir=(\S+)$/) { push @dirs, $1 }
		if ($line =~ /^resource_file=(\S+)$/) { $resource_file = $1 }
	}
	close(FILE);
	foreach my $dir (@dirs) {
		opendir(DIR, $dir) || update_status('error', "error: cannot open $dir to read $!");
		while (my $file = readdir(DIR)) {
			if ($file =~ /(\S+\.cfg$)/) { $file_paths{"$dir/$1"} = $1 }
		}
		close(DIR);
	}
	update_status('info', "Reading: $nagios_home/cgi.cfg");
	open(FILE, "< $nagios_home/cgi.cfg") or update_status('error', "error: cannot open $nagios_home/cgi.cfg to read $!");
	while (my $line = <FILE>) {
		if ($line =~ /^xedtemplate_config_file=(\S+)$/) { push @files, $1 }
	}
	close(FILE);


	foreach my $file (@files) {
		if ($file =~ /.*\/(.*\.cfg)/) { $file_paths{$file} = $1 }
	}
	foreach my $file (keys %file_paths) {
		my ($host_file, $service_file) = 0;
		if ($debug) { print "\nfile: $file\n" }
		if ($file =~ /HASH/) { next }
		my $added = 0;
		my $rejected = 0;
		if ($file =~ /nagios.cfg|resource.cfg|minimal.cfg|perfparse.cfg|cgi.cfg/i) { next }
		my @properties = ();
		open (CFG, "< $file") or update_status('error', "error: cannot open $file $!");
		while (my $line = <CFG>){
			if ($line =~ /#GROUNDWORK#|^#GW|^\s*;/) { next }
			$line =~ s/\t+/ /g;
			$line =~ s/^\s+|\s+$//g;
			my @comment = ();
			my %element = ();
			if ($line =~ /^}/) {
				my @comment = ();
				my %element = ();
				$element{'data'} = "<?xml version=\"1.0\" ?>\n<data>\n";
				foreach my $prop (@properties) {
					if ($prop =~ /^define\s+(\S+)\s*\{/i) { 
						$element{'type'} = $1;
					} elsif ($prop =~ /register\s+0/i) {
						$element{'type'} .= '_template';
					} elsif ($prop =~ /^\#/) {
						push @comment, "$prop\n";			
					} elsif ($prop =~ /(\S+)\s+(\S+.*?);/) {
						my ($name,$value) = ($1,$2);
						$value =~ s/^,|,$//;
						if ($element{$name} eq 'members') {
							$element{$name} .= "$value,";
						} else {
							if ($value eq '0') { $value = "-zero-" }
							$element{$name} = $value;
						}
					} elsif ($prop =~ /(\S+)\s+(\S+.*?)\t/) {
						my ($name,$value) = ($1,$2);
						$value =~ s/^,|,$//;
						$name =~ s/\s+$//;
						if ($element{$name} eq 'members') {
							$element{$name} .= "$value,";
						} else {
							if ($value eq '0') { $value = "-zero-" }
							$element{$name} = $value;
						}
					} elsif ($prop =~ /(\S+)\s+(\S+.*?)$/) {
						my ($name,$value) = ($1,$2);
						$value =~ s/^,|,$//;
						$name =~ s/\s+$//;
						if ($element{$name} eq 'members') {
							$element{$name} .= "$value,";
						} else {
							if ($value eq '0') { $value = "-zero-" }
							$element{$name} = $value;
						}
					}
				}
				if ($element{'type'} eq 'host') {
					$element{'name'} .= $element{'host_name'};
					$element{'parent'} .= $element{'host_name'};
					delete $element{'host_name'};
					if ($file_object{'services'} eq $file_paths{$file}) { 
						my $path = $file;
						$path =~ s/\/$file_paths{$file}//;
						my $f_name = $file_paths{$file};
						$host_file = "$path/hosts_$f_name";
						$file_paths{$host_file} = "hosts_$f_name";
						$element{'data'} .= " <prop name=\"file_name\"><![CDATA[$file_paths{$host_file}]]>\n";
						$element{'data'} .= " </prop>\n";
						$file_type{$file_paths{$host_file}} = 'hosts';
					} else {
						$element{'data'} .= " <prop name=\"file_name\"><![CDATA[$file_paths{$file}]]>\n";
						$element{'data'} .= " </prop>\n";
						$file_type{$file_paths{$file}} = 'hosts';
					}
					$file_object{'hosts'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'service') {
					if (!$element{'service_description'}) { $element{'service_description'} = $element{'use'} }
					$element{'name'} = $element{'service_description'};
					delete $element{'service_description'};
					if ($element{'hostgroup_name'}) {
						my @h = split(/,/, $element{'hostgroup_name'});
						$element{'parent'} = $h[0];
					} elsif ($element{'host_name'}) {
						my @h = split(/,/, $element{'host_name'});
						$element{'parent'} = $h[0];
					}
					if ($file_object{'hosts'} eq $file_paths{$file}) { 
						my $path = $file;
						$path =~ s/\/$file_paths{$file}//;
						my $f_name = $file_paths{$file};
						$service_file = "$path/services_$f_name";
						$file_paths{$service_file} = "services_$f_name";
						$element{'data'} .= " <prop name=\"file_name\"><![CDATA[$file_paths{$service_file}]]>\n";
						$element{'data'} .= " </prop>\n";
						$file_type{$file_paths{$service_file}} = 'services';
					} else {
						$element{'data'} .= " <prop name=\"file_name\"><![CDATA[$file_paths{$file}]]>\n";
						$element{'data'} .= " </prop>\n";
						$file_type{$file_paths{$file}} = 'services';
					}
					$file_object{'services'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'hostgroupescalation') {
					my @name = split(/,/, $element{'hostgroup_name'});
					$element{'name'} = "$name[0]-$index";
					$index++;
				} elsif ($element{'type'} eq 'hostescalation') {
					if ($element{'hostgroup_name'}) {
						my @name = split(/,/, $element{'hostgroup_name'});
						$element{'name'} = "$name[0]-$index";
						$element{'parent'} = "hostgroup";
					} elsif ($element{'host_name'}) {
						my @name = split(/,/, $element{'host_name'});
						$element{'name'} = "$name[0]-$index";
						$element{'parent'} = "host";
					}
					$index++;
				} elsif ($element{'type'} eq 'serviceescalation') {
					if ($element{'hostgroup_name'}) {
						my @name = split(/,/, $element{'hostgroup_name'});
						$element{'name'} = "$name[0]-$index";
						$element{'parent'} = "hostgroup";
					} elsif ($element{'service_description'} && $element{'service_description'} ne '*') {
 						$element{'name'} = "$element{'service_description'}-$index";
						$element{'type'} = 'serviceescalation-service';
						if ($element{'use'}) {
							$element{'parent'} = $element{'use'};
						} else {
							$element{'parent'} = "no-template-$index";
						}
					} elsif ($element{'host_name'}) {
						my @name = split(/,/, $element{'host_name'});
						$element{'name'} = "$name[0]-$index";
						$element{'parent'} = "host";
					}
					$index++;
				} elsif ($element{'type'} eq 'servicedependency') {
					$element{'name'} = "$element{'dependent_service_description'}-$index";
					if ($element{'dependent_hostgroup_name'}) {
						my @h = split(/,/, $element{'dependent_hostgroup_name'});
						$element{'parent'} = "$h[0]-$element{'dependent_service_description'}-$index";
					} elsif ($element{'dependent_host_name'}) {
						my @h = split(/,/, $element{'dependent_host_name'});
						$element{'parent'} = "$h[0]-$element{'dependent_service_description'}-$index";
					}
					$index++;
				} elsif ($element{'type'} eq 'hostdependency') {
					my @h = split(/,/, $element{'dependent_host_name'});
					$element{'name'} = "$h[0]-$index";
					@h = split(/,/, $element{'host_name'});
					$element{'parent'} = $h[0];
					$index++;
				} elsif ($element{'type'} eq 'hostextinfo') {
					my @h = split(/,/, $element{'host_name'});
					if ($h[1]) {
						$element{'name'} = $h[0];
						$element{'parent'} = $element{'type'};
					} else {
						$element{'name'} = $element{'host_name'};
						delete $element{'host_name'};
					}
				} elsif ($element{'type'} eq 'serviceextinfo') {
					$element{'name'} .= $element{'service_description'};
					$element{'parent'} .= $element{'host_name'};
					delete $element{'host_name'};
					delete $element{'service_description'};
				} elsif ($element{'type'} eq 'timeperiod') {
					$element{'name'} .= $element{'timeperiod_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					delete $element{'timeperiod_name'};
				} elsif ($element{'type'} eq 'command') {
					$element{'name'} .= $element{'command_name'};
					$element{'parent'} = $file_paths{$file};
					delete $element{'command_name'};
					if ($element{'command_line'} =~ /\$CONTACTEMAIL\$|\$NOTIFICATIONTYPE\$|\$CONTACTPAGER\$/) {
						$file_object{'miscommands'} = 'misccommands.cfg';
					} else {
						$file_object{'commands'} = 'checkcommands.cfg';
					}
				} elsif ($element{'type'} eq 'contact') {
					$element{'name'} .= $element{'contact_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					delete $element{'contact_name'};
				} elsif ($element{'type'} eq 'contactgroup') {
					$element{'name'} .= $element{'contactgroup_name'};
					delete $element{'contactgroup_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					$element{'members'} =~ s/,$//;
					$file_object{'contactgroups'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'hostgroup') {
					$element{'type'} = 'hostgroup-host';
					$element{'name'} .= $element{'hostgroup_name'};
					delete $element{'hostgroup_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					$element{'members'} =~ s/,$//;
				} elsif ($element{'type'} eq 'servicegroup') {
					$element{'type'} = 'servicegroup';
					$element{'name'} .= $element{'servicegroup_name'};
					delete $element{'servicegroup_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					$element{'members'} =~ s/,$//;
				} elsif ($element{'type'} eq 'service_template') {
					if (!$element{'use'}) { $element{'use'} = 'no-parent' }
					$element{'parent'} = $element{'use'};
					delete $element{'use'};
				}
				if ($element{'type'} =~ /template/ && !$element{'name'}) {
					next;
				}
				foreach my $ele (keys %element) {
					if (!$element{$ele} || $ele =~ /HASH|^name$|^type$|^data$|^parent$/) { next }
					$element{'data'} .= " <prop name=\"$ele\"><![CDATA[$element{$ele}]]>\n";
					$element{'data'} .= " </prop>\n";
				}
				unless ($element{'type'} eq 'service' or $element{'type'} eq 'host') {
					$file_type{$file_paths{$file}} = $element{'type'};
				}
				if (!$element{'parent'}) { $element{'parent'} = $element{'name'} }
				$element{'data'} .= "</data>";
				$element{'name'} = $dbh->quote($element{'name'});
				$element{'type'} = $dbh->quote($element{'type'});
				$element{'parent'} = $dbh->quote($element{'parent'});
				$element{'data'} = $dbh->quote($element{'data'});
				my $comment = pop @comment;
				$comment = $dbh->quote($comment);
# insert 
				my $sqlstmt = "insert into stage_other values($element{'name'},$element{'type'},$element{'parent'},$element{'data'},$comment)";
				$sth = $dbh->prepare($sqlstmt);
				eval{$sth->execute()};					
				if ($@) {
					if ($@ =~ /DBD::mysql::st execute failed: Duplicate entry (\S+) for/) {
						update_status('error', "Duplicate $element{'type'}: $element{'name'} already exists and will be ignored (file: $file).");
						if ($debug) { print "SQL: $sqlstmt $@\n\n----press return to continue:"; my $x = <STDIN>; }
					} else {
						update_status('error', "Error: $file $sqlstmt $@");
					}
					$rejected++;
				} else {
					$added++;
				}
				@properties = ();
				push @properties, $line;
			} else {
				push @properties, $line;
			}
		}
		update_status('info',"File $file: $added objects staged $rejected objects rejected.");
	}
	foreach my $file (sort keys %file_paths) {
		my $path = $file;
		$path =~ s/\/$file_paths{$file}//;
		my @values = ('',$file_paths{$file},$path,$file_type{$file_paths{$file}});
		my $file_id = insert_obj_id('files',\@values,'file_id');
		if ($file_id =~ /Error/) { update_status('error', $file_id) }
		foreach my $obj (keys %file_object) {
			if ($file_object{$obj} eq $file_paths{$file}) {
				@values = ($obj,'file',$file_id);
				insert_obj('setup',\@values);
				
				delete $file_object{$obj};
			}
		}
	}
	foreach my $obj (keys %file_object) {
		my @values = ('',$file_object{$obj},$nagios_home,$obj);
		my $file_id = insert_obj_id('files',\@values,'file_id');
		if ($file_id =~ /Error/) { update_status('error', $file_id) }
		@values = ($obj,'file',$file_id);
		insert_obj('setup',\@values);
	}
	foreach (@results) { update_status('info',$_) }
	$dt = datetime();
	update_status('info',"$dt Stage completed.");

	update_status('info', "Reading: $resource_file");
	my %w = ('name' => 'resource_home');
	my %u = ('value' => $resource_file);
	update_obj_where('setup',\%u,\%w);
	
	open(FILE, "< $resource_file") or update_status('error', "error: cannot open $resource_file to read $!");
	while (my $line = <FILE>) {
		if ($line =~ /\$USER(\d+)\$=(\S+)$/) { 
			if ($1 <= 32) {
				my %w = ('name' => "user$1");
				my %u = ('value' => $2);
				update_obj_where('setup',\%u,\%w);
			}
		}
	}
	close(FILE);
	update_status('end',"$dt Stage completed.");
	return @messages;
}

#
###########################################################################################################################
# Process
#

sub process_objects() {
	my $dt = datetime();
	update_status('start',"$dt Processing.");
# Set special * service
	my %splat = fetch_one('service_names','name','*');
	my $splat_id = $splat{'servicename_id'};
	unless ($splat_id) {
		my @values = ('','*','special use','','','','','','');
		$splat_id = insert_obj_id('service_names',\@values,'servicename_id');
	}
# commands
	update_status('info',"Loading commands");
	my $type = 'command';
	my %where = ('type' => $type);
	my %objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my $cmdtype = 'check';
		if ($data{'command_line'} =~ /\$CONTACTEMAIL\$|\$NOTIFICATIONTYPE\$|\$CONTACTPAGER\$/) {
			$cmdtype = 'notify';
		} elsif ($data{'command_line'} =~ /\$PERFDATA\$/) {
			$cmdtype = 'other';
		}
		my $data = $xml_begin;
		$data .= " <prop name=\"command_line\"><![CDATA[$data{'command_line'}]]>\n";
		$data .= " </prop>\n";
		$data .= $xml_end;
		my %obj = fetch_one('commands','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			my %values = ();
			$values{'type'} = $cmdtype;
			$values{'data'} = $data;
			$values{'comment'} = $objects{$key}[4];
			update_obj('commands','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$cmdtype,$data,$objects{$key}[4]);
			insert_obj('commands',\@values);
		}
	}
	delete_one_where('stage_other',\%where);
	

# timeperiods
	update_status('info',"Loading timeperiods");
	$type = 'timeperiod';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %obj = fetch_one('time_periods','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			my %values = ();
			$values{'alias'} = $objects{$key}[2];
			$values{'data'} = $objects{$key}[3];
			$values{'comment'} = $objects{$key}[4];
			update_obj('time_periods','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$objects{$key}[2],$objects{$key}[3],$objects{$key}[4]);
			insert_obj('time_periods',\@values);
		}
	}
	delete_one_where('stage_other',\%where);
	

# contact template
	update_status('info',"Loading contact templates");
	$type = 'contact_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my %h = fetch_one('time_periods','name',$data{'host_notification_period'});
		my %s = fetch_one('time_periods','name',$data{'service_notification_period'});
		delete $data{'host_notification_period'};
		delete $data{'service_notification_period'};
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /command/) { next }
			$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
			$data .= " </prop>\n";
		}
		$data .= $xml_end;				
		my $id = undef;
		my %obj = fetch_one('contact_templates','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			$id = $obj{'contacttemplate_id'};
			my %values = ();
			$values{'host_notification_period'} = $h{'timeperiod_id'};
			$values{'service_notification_period'} = $s{'timeperiod_id'};
			$values{'comment'} = $objects{$key}[4];
			update_obj('contact_templates','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$h{'timeperiod_id'},$s{'timeperiod_id'},$data,$objects{$key}[4]);
			$id = insert_obj_id('contact_templates',\@values,'contacttemplate_id');
		}
		delete_all('contact_command','contacttemplate_id',$id);
		my @h = split(/,/, $data{'host_notification_commands'});		
		foreach my $h (@h) {
			$h =~ s/^\s+|\s+$//;
			my %c = fetch_one('commands','name',$h);
			if (!$c{'command_id'}) { next }
			my @vals = ($id,'host',$c{'command_id'});
			insert_obj('contact_command',\@vals);
		}
		my @s = split(/,/, $data{'service_notification_commands'});		
		foreach my $s (@s) {
			$s =~ s/^\s+|\s+$//;
			my %c = fetch_one('commands','name',$s);
			if (!$c{'command_id'}) { next }
			my @vals = ($id,'service',$c{'command_id'});
			insert_obj('contact_command',\@vals);
		}
	}
	delete_one_where('stage_other',\%where);
	

# contacts
	update_status('info',"Loading contacts");
	$type = 'contact';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	my $index = 1;
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my %hostnotify = fetch_one('time_periods','name',$data{'host_notification_period'});
		my %servicenotify = fetch_one('time_periods','name',$data{'service_notification_period'});
		$data{'host_notification_period'} = $hostnotify{'timeperiod_id'};
		$data{'service_notification_period'} = $servicenotify{'timeperiod_id'};
		my @h = split(/,/, $data{'host_notification_commands'});
		my %h_commands = ();
		foreach my $h (@h) {
			$h =~ s/^\s+|\s+$//;
			my %c = fetch_one('commands','name',$h);
			$h_commands{$h} = $c{'command_id'};
		}
		my @s = split(/,/, $data{'service_notification_commands'});
		my %s_commands = ();
		foreach my $s (@s) {
			$s =~ s/^\s+|\s+$//;
			my %c = fetch_one('commands','name',$s);
			$s_commands{$s} = $c{'command_id'};
		}
		if (!$data{'use'}) {
			update_status('note', "Contact $objects{$key}[0] does not use a template.");
			my $template_id = undef;
			my $bestmatch = 0;
			my %w = ();
			my %templates = fetch_list_hash_array('contact_templates',\%w);
			foreach my $t (keys %templates) {
				my $gotmatch = 0;
				my %w = ('contacttemplate_id' => $t);
				my @tmplate_commands = fetch_list_where('contact_command','command_id',\%w);
				foreach my $tc (@tmplate_commands) {
					foreach my $key (keys %h_commands) {
						if ($h_commands{$key} eq $tc) { $gotmatch++ }
					}
					foreach my $key (keys %s_commands) {
						if ($s_commands{$key} eq $tc) { $gotmatch++ }
					}
				}
				my %tdata = parse_xml($templates{$t}[4]);
				if ($data{'host_notification_period'} eq $templates{$t}[2]) { $gotmatch++ }
				if ($data{'service_notification_period'} eq $templates{$t}[3]) { $gotmatch++ }
				foreach my $d (keys %data) {
					unless ($d =~ /email|pager|period|command/) { 
						if ($data{$d} eq $tdata{$d}) { $gotmatch++ }
					}
				}		
				if ($gotmatch > $bestmatch) {
					$bestmatch = $gotmatch;
					$data{'use'} = $templates{$t}[1];
					$template_id = $templates{$t}[0];
				}
			}
			if ($data{'use'}) {
				update_status('action', "Assigned: Contact template $data{'use'} assigned to contact $objects{$key}[0]");
			}
		} 
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /email|pager|period|command/) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}
			$data .= $xml_end;				
			my $name = "generic-contact-$index";
			my $comment = undef;
			my @values = ('',$name,$data{'host_notification_period'},$data{'service_notification_period'},$data,$comment);
			my $id = insert_obj_id('contact_templates',\@values,'contacttemplate_id');
			foreach my $key (keys %h_commands) {
				if (!$h_commands{$key}) { next }
				my @vals = ($id,'host',$h_commands{$key});
				insert_obj('contact_command',\@vals);
			}
			foreach my $key (keys %s_commands) {
				if (!$s_commands{$key}) { next }
				my @vals = ($id,'service',$s_commands{$key});
				insert_obj('contact_command',\@vals);
			}
			update_status('action', "Added: Contact template $name, assigned to contact $objects{$key}[0]");
			$data{'use'} = $name;
			$index++;
		}
		my $id = undef;
		my %t = fetch_one('contact_templates','name',$data{'use'});
		my %obj = fetch_one('contacts','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			$id = $obj{'contact_id'};
			my %values = ();
			$values{'alias'} = $objects{$key}[2];
			$values{'email'} = $data{'email'};
			$values{'pager'} = $data{'pager'};
			$values{'contacttemplate_id'} = $t{'contacttemplate_id'};
			$values{'status'} = '1';
			$values{'comment'} = $objects{$key}[4];
			update_obj('contacts','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$objects{$key}[2],$data{'email'},$data{'pager'},$t{'contacttemplate_id'},'1',$objects{$key}[4]);
			$id = insert_obj_id('contacts',\@values,'contact_id');
		}
		# process overrides
		my %w = ('contacttemplate_id' => $t{'contacttemplate_id'},'type' => 'host');
		my @tmplate_commands = fetch_list_where('contact_command','command_id',\%w);
		delete_all('contact_overrides','contact_id',$id);
		delete_all('contact_command_overrides','contact_id',$id);
		my $match = 0;
		my $i = 0;
		foreach my $tc (@tmplate_commands) {
			$i++;
			foreach my $key (keys %h_commands) {
				unless ($h_commands{$key} eq $tc) { $match++ }
			}
		}
		if ($i == $match) {
			foreach my $k (keys %h_commands) {
				my @values = ($id,'host',$h_commands{$k});
				insert_obj('contact_command_overrides',\@values);
			}
		}
		my %w = ('contacttemplate_id' => $t{'contacttemplate_id'},'type' => 'service');
		my @tmplate_commands = fetch_list_where('contact_command','command_id',\%w);
		$match = 0;
		$i = 0;
		foreach my $tc (@tmplate_commands) {
			$i++;
			foreach my $key (keys %s_commands) {
				if ($s_commands{$key} eq $tc) { $match++ }
			}
		}
		if ($i == $match) {
			foreach my $k (keys %s_commands) {
				my @values = ($id,'service',$s_commands{$k});
				insert_obj('contact_command_overrides',\@values);
			}
		}
		my %overrides = ();
		my @sort = split(/,/, $data{'host_notification_period'});
		@sort = sort @sort;
		my $sorted = undef;
		foreach (@sort) { $sorted .= "$_,"  }
		chop $sorted;
		$data{'host_notification_period'} = $sorted;
		@sort = split(/,/, $data{'service_notification_period'});
		@sort = sort @sort;
		$sorted = undef;
		foreach (@sort) { $sorted .= "$_,"  }
		chop $sorted;
		$data{'service_notification_period'} = $sorted;
		my @sort = split(/,/, $t{'host_notification_period'});
		@sort = sort @sort;
		my $sorted = undef;
		foreach (@sort) { $sorted .= "$_,"  }
		chop $sorted;
		$t{'host_notification_period'} = $sorted;
		@sort = split(/,/, $t{'service_notification_period'});
		@sort = sort @sort;
		$sorted = undef;
		foreach (@sort) { $sorted .= "$_,"  }
		chop $sorted;
		$t{'service_notification_period'} = $sorted;
		unless ($data{'host_notification_period'} eq $t{'host_notification_period'}) { $overrides{'host_notification_period'} = $data{'host_notification_period'} }
		unless ($data{'service_notification_period'} eq $t{'service_notification_period'}) { $overrides{'service_notification_period'} = $data{'service_notification_period'} }
		unless ($data{'host_notification_options'} eq $t{'host_notification_options'}) { $overrides{'host_notification_options'} = $data{'host_notification_options'} }
		unless ($data{'service_notification_options'} eq $t{'service_notification_options'}) { $overrides{'service_notification_options'} = $data{'service_notification_options'} }
		my $data = undef;
		foreach my $d (keys %overrides) {
			if ($d =~ /options/) {
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}
		}
		if ($data) { $data = $xml_begin.$data.$xml_end }
		if ($data{'service_notification_options'} || $data{'service_notification_period'} || $overrides{'host_notification_period'} || $overrides{'service_notification_period'}) {
			my @values = ($id,$overrides{'host_notification_period'},$overrides{'service_notification_period'},$data);
			insert_obj('contact_overrides',\@values);
		}
	}
	delete_one_where('stage_other',\%where);
	

# contactgroups
	update_status('info',"Loading contactgroups");
	$type = 'contactgroup';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my $id = undef;
		my %obj = fetch_one('contactgroups','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			$id = $obj{'contactgroup_id'};
			my %values = ();
			$values{'alias'} = $objects{$key}[2];
			$values{'comment'} = $objects{$key}[4];
			update_obj('contactgroups','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$objects{$key}[2],$objects{$key}[4]);
			$id = insert_obj_id('contactgroups',\@values,'contactgroup_id');
			if ($id =~ /Error/) { update_status('error',$id) }
		}
		delete_all('contactgroup_contact','contactgroup_id',$id);
		my %data = parse_xml($objects{$key}[3]);
		my %members = ();
		my @mems = split(/,/, $data{'members'});
		foreach (@mems) { $members{$_} = 1 }
		foreach my $mem (keys %members) {
			$mem =~ s/^\s+|\s+$//;
			my %cid = fetch_one('contacts','name',$mem);
			if ($cid{'contact_id'}) {
				my @vals = ($id,$cid{'contact_id'});
				insert_obj('contactgroup_contact',\@vals);
			} else {
				if ($debug) { print "\nError: $mem is not a valid contact so will be ignored.\n" }
				update_status('info',"$mem is not a valid contact so will be ignored.");
			}
		}
	}
	delete_one_where('stage_other',\%where);
	

# host templates
	update_status('info',"Loading host templates");
	$type = 'host_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my $props = 0;
		foreach (keys %data) { $props++ }
		my $data = undef;
		if ($data{'parents'}) { 
			$props -= 1;
			$data = $xml_begin;
			$data .= " <prop name=\"parents\"><![CDATA[$data{'parents'}]]>\n";		
			$data .= " </prop>\n";
			$data .= $xml_end;
		}
		if ($data) {
			my @vals = ($objects{$key}[0],'parent_directive',$data{'use'},$data,'');
			insert_obj('stage_other',\@vals);
			update_status('note', "Parent directive on host template $objects{$key}[0].");
			update_status('action', "Parent directive moved: Parents $data{'parents'} moved from host template $objects{$key}[0] to hosts.");
		}
		if ($data{'use'} && $props == 1) {
			my @vals = ($objects{$key}[0],'redundant_template',$data{'use'},$data,'');
			insert_obj('stage_other',\@vals);
			update_status('note', "Redundant: Host template $objects{$key}[0] is redundant as it references $data{'use'} but has no properties itself.");
			update_status('action', "Staged for removal: Host template $objects{$key}[0]. Hosts using $objects{$key}[0] will be referenced to $data{'use'}.");
		} elsif ($data{'use'}) {
			my %w = ('type' => 'host_template','name' => $data{'use'});
			my %parent = fetch_one_where('stage_other',\%w);
			my %pdata = parse_xml($parent{'data'});
			foreach my $p (keys %pdata) {
				unless ($data{$p}) { $data{$p} = $pdata{$p} }
			}
			update_status('note', "Host template $objects{$key}[0] is a host template that references a host template.");
			update_status('action', "Reference removed: Host template $objects{$key}[0] will no longer reference to $data{'use'}. Please check Manage->Host Templates->$objects{$key}[0] values.");
		}
		my %t = fetch_one('time_periods','name',$data{'notification_period'});
		my %cp = fetch_one('time_periods','name',$data{'check_period'});
		my %c = fetch_one('commands','name',$data{'check_command'});
		my %e = fetch_one('commands','name',$data{'event_handler'});
		delete $data{'notification_period'};
		delete $data{'check_command'};
		delete $data{'event_handler'};
		$data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /parents|contact_groups|use/) { next }
			$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
			$data .= " </prop>\n";
		}	
		$data .= $xml_end;
		my $id = undef;
		my %obj = fetch_one('host_templates','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			$id = $obj{'hosttemplate_id'};
			my %values = ();
			$values{'notification_period'} = $t{'timeperiod_id'};
			$values{'check_command'} = $c{'command_id'};
			$values{'event_handler'} = $e{'command_id'};
			$values{'data'} = $data;
			$values{'comment'} = $objects{$key}[4];
			update_obj('host_templates','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$cp{'timeperiod_id'},$t{'timeperiod_id'},$c{'command_id'},$e{'command_id'},$data,$objects{$key}[4]);
			$id = insert_obj_id('host_templates',\@values,'hosttemplate_id');
		}
		my %w = ('type' => 'host_templates','object' => $id);
		delete_one_where('contactgroup_assign',\%w);
		my @members = split(/,/, $data{'contact_groups'});
		foreach my $mem (@members) {
			$mem =~ s/^\s+|\s+$//;
			my %c = fetch_one('contactgroups','name',$mem);
			unless ($c{'contactgroup_id'}) { 
				update_status('error', "Contact group $mem does not exist for host template $objects{$key}[0].");
			} else {
				my @vals = ($c{'contactgroup_id'},'host_templates',$id);
				insert_obj('contactgroup_assign',\@vals);
			}
		}
	}
	delete_one_where('stage_other',\%where);
	

# service templates
	update_status('info',"Loading service templates");
	%where = ('type' => 'service_template');
	%objects = fetch_list_hash_array('stage_other',\%where);
	my %parents = ();
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		if ($debug) {
			print "\n----- $objects{$key}[0]\n";
		}
		my %t = fetch_one('time_periods','name',$data{'check_period'});
		my %n = fetch_one('time_periods','name',$data{'notification_period'});
		my %c = fetch_one('commands','name',$data{'check_command'});
		my %e = fetch_one('commands','name',$data{'event_handler'});
		delete $data{'check_period'};
		delete $data{'notification_period'};
		delete $data{'check_command'};
		delete $data{'event_handler'};
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /contact_groups/) { next }
			$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
			$data .= " </prop>\n";
		}	
		$data .= $xml_end;
		my $id = undef;
		my %obj = fetch_one('service_templates','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			$id = $obj{'servicetemplate_id'};
			my %values = ();
			my %p = fetch_one('service_templates','name',$objects{$key}[2]);
			$values{'parent_id'} = $p{'servicetemplate_id'};
			$values{'check_period'} = $t{'timeperiod_id'};
			$values{'notification_period'} = $n{'timeperiod_id'};
			$values{'check_command'} = $c{'command_id'};
			$values{'event_handler'} = $e{'command_id'};
			$values{'data'} = $data;
			$values{'comment'} = $objects{$key}[4];
			update_obj('service_templates','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],'',$t{'timeperiod_id'},$n{'timeperiod_id'},$c{'command_id'},$e{'command_id'},$data,$objects{$key}[4]);
			$id = insert_obj_id('service_templates',\@values,'servicetemplate_id');
			unless ($objects{$key}[2] eq '') { $parents{$id} = $objects{$key}[2] }
		}	
		my %w = ('type' => 'service_templates','object' => $id);
		delete_one_where('contactgroup_assign',\%w);
		my @members = split(/,/, $data{'contact_groups'});
		foreach my $mem (@members) {
			$mem =~ s/^\s+|\s+$//;
			my %c = fetch_one('contactgroups','name',$mem);
			unless ($c{'contactgroup_id'}) { 
				update_status('error', "Contact group $mem does not exist for service template $objects{$key}[0]");
			} else {
				my @vals = ($c{'contactgroup_id'},'service_templates',$id);
				insert_obj('contactgroup_assign',\@vals);
			}
		}
	}
	delete_one_where('stage_other',\%where);
	foreach my $id (keys %parents) {
		my %p = fetch_one('service_templates','name',$parents{$id});
		my %values = ('parent_id' => $p{'servicetemplate_id'});
		update_obj('service_templates','servicetemplate_id',$id,\%values);
	}

# hostextinfo templates
	update_status('info',"Loading hostextinfo templates");
	$type = 'hostextinfo_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	$index = 1;
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my %obj = fetch_one('extended_host_info_templates','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			my %values = ();
			$values{'data'} = $objects{$key}[3];
			$values{'comment'} = $objects{$key}[4];
			update_obj('extended_host_info_templates','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$objects{$key}[3],'',$objects{$key}[4]);
			insert_obj('extended_host_info_templates',\@values);
		}
	}
	delete_one_where('stage_other',\%where);
	

# serviceextinfo templates
	update_status('info',"Loading serviceextinfo templates");
	$type = 'serviceextinfo_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %obj = fetch_one('extended_service_info_templates','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			my %values = ();
			$values{'data'} = $objects{$key}[3];
			$values{'comment'} = $objects{$key}[4];
			update_obj('extended_service_info_templates','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$objects{$key}[3],'',$objects{$key}[4]);
			insert_obj('extended_service_info_templates',\@values);
		}
	}
	delete_one_where('stage_other',\%where);
	

# hostgroupescalation templates
	update_status('info',"Loading hostgroupescalation templates");
	$type = 'hostgroupescalation_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgids;
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /host|name|type|HASH|comment/i) { next } 
			if ($d =~ /contact_groups/) { 
				my @cgs = split(/,/, $data{'contact_groups'});
				foreach my $c (@cgs) {
					$c =~ s/^\s+|\s+$//;
					my %cg = fetch_one('contactgroups','name',$c);
					unless ($cg{'contactgroup_id'}) { 
						update_status('error', "Contact group $c does not exist for hostgroup escalation template $objects{$key}[0].");
					} else {
						push @cgids, $cg{'contactgroup_id'};
					}
				}
			} else {
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}
		}	
		$data .= $xml_end;
		my $id = undef;
		my %where = ('name' => $objects{$key}[0],'type' => 'hostgroup');
		my %obj = fetch_one_where('escalation_templates',\%where);
		if ($obj{'name'}) {
			$id = $obj{'template_id'};
			my %values = ();
			$values{'data'} = $data;
			$values{'comment'} = $objects{$key}[4];
			$values{'servicename_id'} = $splat_id;
			update_obj('escalation_templates','template_id',$id,\%values);
		} else {
			my @values = ('',$objects{$key}[0],'hostgroup',$data,$objects{$key}[4],$splat_id);
			$id = insert_obj_id('escalation_templates',\@values,'template_id');
		}
		my %w = ('type' => 'hostgroup_escalation_templates','object' => $id);
		delete_one_where('contactgroup_assign',\%w);
		foreach my $cgid (@cgids) {
			my @vals = ($cgid,'hostgroup_escalation_templates',$id);
			insert_obj('contactgroup_assign',\@vals);
		}
	}
	delete_one_where('stage_other',\%where);
	

# hostescalation templates
	update_status('info',"Loading hostescalation templates");
	$type = 'hostescalation_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgids = ();
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /host|name|type|HASH|comment/i) { next } 
			if ($d =~ /contact_groups/) { 
				my @cgs = split(/,/, $data{'contact_groups'});
				foreach my $c (@cgs) {
					$c =~ s/^\s+|\s+$//;
					my %cg = fetch_one('contactgroups','name',$c);
					unless ($cg{'contactgroup_id'}) { 
						update_status('error', "Contact group $c does not exist for host escalation template $objects{$key}[0].");
					} else {
						push @cgids, $cg{'contactgroup_id'};
					}
				}
			} else {
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}
		}	
		$data .= $xml_end;
		my $id = undef;
		my %w = ('name' => $objects{$key}[0],'type' => 'host');
		my %obj = fetch_one_where('escalation_templates',\%w);
		if ($obj{'name'}) {
			$id = $obj{'template_id'};
			my %values = ();
			$values{'data'} = $data;
			$values{'comment'} = $objects{$key}[4];
			$values{'servicename_id'} = $splat_id;
			update_obj('escalation_templates','template_id',$id,\%values);
		} else {
			my @values = ('',$objects{$key}[0],'host',$data,$objects{$key}[4],$splat_id);
			$id = insert_obj_id('escalation_templates',\@values,'template_id');
			if ($id =~ /Error/) { update_status('error',$id) }
		}
		my %w = ('type' => 'host_escalation_templates','object' => $id);
		delete_one_where('contactgroup_assign',\%w);	
		foreach my $cgid (@cgids) {
			my @vals = ($cgid,'host_escalation_templates',$id);
			insert_obj('contactgroup_assign',\@vals);
		}
	}
	

# serviceescalation templates
	update_status('info',"Loading serviceescalation templates");
	$type = 'serviceescalation_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		$objects{'service_description'} = 0;
		my %data = parse_xml($objects{$key}[3]);
		my @cgids = ();
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /host|name|type|HASH|comment/i) { next } 
			if ($d eq 'service_description') {
				my %s = fetch_one('service_names','name',$data{$d});
				$objects{$d} = $s{'servicename_id'};
			} elsif ($d =~ /contact_groups/) { 
				my @cgs = split(/,/, $data{'contact_groups'});
				foreach my $c (@cgs) {
					$c =~ s/^\s+|\s+$//;
					my %cg = fetch_one('contactgroups','name',$c);
					unless ($cg{'contactgroup_id'}) { 
						update_status('error', "Contact group $c does not exist for service escalation template $objects{$key}[0].");
					} else {
						push @cgids, $cg{'contactgroup_id'};
					}
				}
			} else {
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}
		}	
		$data .= $xml_end;
		my %sn = fetch_one('service_names','name',$objects{'service_description'});
		my $id = undef;
		my %w = ('name' => $objects{$key}[0],'type' => 'service');
		my %obj = fetch_one_where('escalation_templates',\%w);
		if ($obj{'name'}) {
			$id = $obj{'template_id'};
			my %values = ();
			$values{'data'} = $data;
			$values{'comment'} = $objects{$key}[4];
			$values{'servicename_id'} = $sn{'servicename_id'};
			update_obj('escalation_templates','template_id',$id,\%values);
		} else {
			my @values = ('',$objects{$key}[0],'service',$data,$objects{$key}[4],$sn{'servicename_id'});
			$id = insert_obj_id('escalation_templates',\@values,'template_id');
			if ($id =~ /Error/) { update_status('error',$id) }
		}
		my %w = ('type' => 'service_escalation_templates','object' => $id);
		delete_one_where('contactgroup_assign',\%w);
		foreach my $cgid (@cgids) {
			my @vals = ($cgid,'service_escalation_templates',$id);
			insert_obj('contactgroup_assign',\@vals);
		}
	}
	delete_one_where('stage_other',\%where);
	return @messages;
}

# hosts
sub process_hosts() {
	update_status('info',"Loading hosts");
	my %host_parents = ();
	my $type = 'host';
	my %where = ('type' => $type);
	my %objects = fetch_list_hash_array('stage_other',\%where);
	my $index = 1;
	foreach my $key (keys %objects) {
		my $status = 1;
		my %host_overrides = ();
		my %data = parse_xml($objects{$key}[3]);
		delete $data{'command_line'};
		my %contactgroups = ();
		my @parents = ();
		if ($data{'parents'}) {
			@parents = split(/,/, $data{'parents'});
			delete $data{'parents'};
		}
		if ($data{'check_period'}) { 
			my %check_period = fetch_one('time_periods','name',$data{'check_period'});
			if ($check_period{'timeperiod_id'}) {
				$host_overrides{'check_period'} = $check_period{'timeperiod_id'};
			} else {
				my $tname = $data{'check_period'};
				update_status('error', "Time period $data{'check_period'} is specified for host $objects{$key}[0] but no such time period exists.");
				my @values = ('',$data{'check_period'},'',$xml_begin.$xml_end,'');
				$data{'check_period'} = insert_obj_id('time_periods',\@values,'timeperiod_id');
				update_status('action', "Added time period $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
			}	
			delete $data{'check_period'};
		}
		if ($data{'notification_period'}) { 
			my %notification_period = fetch_one('time_periods','name',$data{'notification_period'});
			if ($notification_period{'timeperiod_id'}) {
				$host_overrides{'notification_period'} = $notification_period{'timeperiod_id'};
			} else {
				my $tname = $data{'notification_period'};
				update_status('error', "Time period $data{'notification_period'} is specified for host $objects{$key}[0] but no such time period exists.");
				my @values = ('',$data{'notification_period'},'',$xml_begin.$xml_end,'');
				$host_overrides{'notification_period'} = insert_obj_id('time_periods',\@values,'timeperiod_id');
				update_status('action', "Added time period $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
			}	
			delete $data{'notification_period'};
		}
		if ($data{'check_command'}) {
			my %check_command = fetch_one('commands','name',$data{'check_command'});
			if ($check_command{'command_id'}) {
				$host_overrides{'check_command'} = $check_command{'command_id'}
			} else {
				my $tname = $data{'check_command'};
				update_status('error', "Check command $data{'check_command'} is specified for host $objects{$key}[0] but no such command definition exists.");
				my @values = ('',$data{'check_command'},'check',$xml_begin.$xml_end,'');
				$host_overrides{'check_command'} = insert_obj_id('commands',\@values,'command_id');
				update_status('action', "Added check command $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
			}
			delete $data{'check_command'};
		}
		if ($data{'event_handler'}) { 
			my %event_handler = fetch_one('commands','name',$data{'event_handler'});
			if ($event_handler{'command_id'}) {
				$host_overrides{'event_handler'} = $event_handler{'command_id'}
			} else {
				my $tname = $data{'event_handler'};
				update_status('error', "Event handler $data{'event_handler'} is specified for host $objects{$key}[0] but no such command definition exists.");
				my @values = ('',$data{'event_handler'},'check',$xml_begin.$xml_end,'');
				$host_overrides{'event_handler'} = insert_obj_id('commands',\@values,'command_id');
				update_status('action', "Added event handler $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
			}
			delete $data{'event_handler'};
		}
		if ($data{'contact_groups'}) { 
			my @cgs = split(/,/, $data{'contact_groups'});
			foreach (@cgs) {
				my %c = fetch_one('contactgroups','name',$_);
				$contactgroups{$_} = 1;
			}
			delete $data{'contact_groups'};
		}

		foreach my $k (keys %data) {
			$host_overrides{$k} = $data{$k};
		}

		unless ($data{'use'}) {
			update_status('note', "Host $objects{$key}[0] does not use a template");
			my %w = ();
			my %templates = fetch_list_hash_array('host_templates',\%w);
			my $bestmatch = 0;
			my $tname = undef;
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[6]);
				my $gotmatch = 0;
				if ($host_overrides{'check_period'} eq $templates{$t}[2]) { $gotmatch++ }
				if ($host_overrides{'notification_period'} eq $templates{$t}[3]) { $gotmatch++ }
				if ($host_overrides{'check_command'} eq $templates{$t}[4]) { $gotmatch++ }
				if ($host_overrides{'event_handler'} eq $templates{$t}[5]) { $gotmatch++ }
				foreach my $d (keys %data) {
					if ($d =~ /file_name|address|alias|parents/) { next }
					if ($data{$d} eq $tdata{$d}) { $gotmatch++ }
				}	
				my %w = ('type' => 'host_templates','object' => $templates{'hosttemplate_id'});
				my @cgids = fetch_list_where('contactgroup_assign','contactgroup_id',\%w);
				foreach (@cgids) {
					foreach my $cg (keys %contactgroups) {
						if ($contactgroups{$cg} eq $_) { $gotmatch++ }
					}
				}
				if ($gotmatch > $bestmatch) {
					$bestmatch = $gotmatch;
					$data{'use'} = $templates{$t}[0];
					$tname = $templates{$t}[1];
				}
			}
			update_status('action', "Assigned host template $tname to host $objects{$key}[0].");
		} else {
			my %template = fetch_one('host_templates','name',$data{'use'});
			if ($template{'hosttemplate_id'}) {
				$data{'use'} = $template{'hosttemplate_id'};
			} else {
				my $tname = $data{'use'};
				update_status('error', "host $objects{$key}[0] specifies template $tname but no such template exists.");
				my $data = $xml_begin.$xml_end;
				my $comment = undef;
				my @values = ('',$tname,'','','','',$data,$comment);
				$data{'use'} = insert_obj_id('host_templates',\@values,'hosttemplate_id');
				update_status('action', "Added host template $tname, but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
			}
		}
		my %template = fetch_one('host_templates','hosttemplate_id',$data{'use'});
		my $id = undef;
		my %obj = fetch_one('hosts','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			$id = $obj{'host_id'};
			my %values = ();
			$values{'alias'} = $data{'alias'};
			$values{'address'} = $data{'address'};
			$values{'hosttemplate_id'} = $data{'use'};
			$values{'comment'} = $objects{$key}[4];
			update_obj('hosts','name',$objects{$key}[0],\%values);
		} else {
			my @values = ('',$objects{$key}[0],$data{'alias'},$data{'address'},'n/a',$data{'use'},'','','','','',$status,$objects{$key}[4]);
			$id = insert_obj_id('hosts',\@values,'host_id');
			if ($id =~ /Error/) { update_status('error',$id) }
		}
		delete_all('host_overrides','host_id',$id);
		foreach my $k (keys %template) {
			if ($template{$k} eq $host_overrides{$k}) { delete $host_overrides{$k} }
		}
		if ($host_overrides{'check_period'} || $host_overrides{'notification_period'} || $host_overrides{'check_command'} || $host_overrides{'event_handler'} || $host_overrides{'data'}) {
			my $data = $xml_begin;
			foreach my $o (keys %host_overrides) {
				unless ($o =~ /alias|address|file_name|check_command|^use$/) {
					$data .= " <prop name=\"$o\"><![CDATA[$host_overrides{$o}]]>\n";				
					$data .= " </prop>\n";
				}
			}
			$data .= $xml_end;
			my @values = ($id,$host_overrides{'check_period'},$host_overrides{'notification_period'},$host_overrides{'check_command'},$host_overrides{'event_handler'},$data);
			insert_obj('host_overrides',\@values);
		}
		my @contactgroups = ();
		my %w = ('type' => 'host_templates','object' => $template{'hosttemplate_id'});
		my @cgids = fetch_list_where('contactgroup_assign','contactgroup_id',\%w);
		foreach my $c (keys %contactgroups) {
			push @contactgroups, $c;
			foreach (@cgids) {
				if (exists $contactgroups{$_}) { delete $contactgroups{$_}	}
			}
		}
		my $got_group = 0;
		foreach my $k (keys %contactgroups) { $got_group = 1 }
		my %w = ('type' => 'hosts','object' => $id);
		delete_one_where('contactgroup_assign',\%w);
		if ($got_group) {
			foreach my $cg (@contactgroups) {
				my %cgid = fetch_one('contactgroups','name',$cg);
				my @vals = ($cgid{'contactgroup_id'},'hosts',$id);
				insert_obj('contactgroup_assign',\@vals);		
			}
		}
		foreach my $p (@parents) {
			$host_parents{$id} .= "$p,";
		}
		
		delete_all('file_host','host_id',$id);
		my %f = fetch_one('files','name',$data{'file_name'});
		my @vals = ($f{'file_id'},$id);
		insert_obj('file_host',\@vals);
		
	}
	delete_one_where('stage_other',\%where);


# host parent
	update_status('info',"Setting host parent relationships");
	if ($debug) { print "\nSetting host parent relationships...\n" }
	truncate_table('host_parent');
	foreach my $host (keys %host_parents) {
		my @parents = split(/,/, $host_parents{$host});
		foreach my $p (@parents) {
			$p =~ s/^\s+|\s+$//;
			my %p = fetch_one('hosts','name',$p);
			my @vals = ($host,$p{'host_id'});
			insert_obj('host_parent',\@vals);	
		}
	}
	
# host dependency
	update_status('info',"Loading host dependencies");
	truncate_table('host_dependencies');
	$type = 'hostdependency';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @hosts = split(/,/, $data{'dependent_host_name'});
		my @parents = split(/,/, $data{'host_name'});
		my %parent_host = ();
		foreach my $host (@hosts) {
			$host =~ s/^\s+|\s+$//;
			my %h = fetch_one('hosts','name',$host);
			foreach my $parent (@parents) {
				my %p = fetch_one('hosts','name',$parent);
				$parent_host{$h{'host_id'}} = $p{'host_id'};
			}
		}		
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /host_name/) { next }
			$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
			$data .= " </prop>\n";
		}	
		$data .= $xml_end;
		foreach my $h (keys %parent_host) {
			my @values = ($h,$parent_host{$h},$data,$objects{$key}[4]);
			insert_obj('host_dependencies',\@values);
		}
	}
	delete_one_where('stage_other',\%where);

# hostextinfo
	update_status('info',"Loading hostextinfo");
	$type = 'hostextinfo';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	$index = count('extended_host_info_templates');
	foreach my $key (keys %objects) {
		my @hosts = ();
		my $hoststr = undef;
		my %data = parse_xml($objects{$key}[3]);
		if ($objects{$key}[2] eq 'hostextinfo') {
			@hosts = split(/,/, $data{'host_name'});
		} else {
			push @hosts, $objects{$key}[0];
		}
		if ($data{'notes_url'}) { 
			if ($hosts[0] && $hosts[0] ne '?') { $data{'notes_url'} =~ s/$hosts[0]/\$HOSTNAME\$/g }
			$data{'notes_url'} =~ s/\s/+/g;
		}
		if (!$data{'use'}) {
			my $props = 0;
			foreach (keys %data) { $props++ }
			if ($data{'host_name'}) { $props -= 1 }
			if ($data{'notes_url'}) { $props -= 1 }
			if ($data{'2d_coords'}) { $props -= 1 }
			if ($data{'3d_coords'}) { $props -= 1 }
			my %w = ();
			my %templates = fetch_list_hash_array('extended_host_info_templates',\%w);
			my %gotmatch = ();
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[2]);
				foreach my $d (keys %data) {
					if ($d =~ /d_coords|host_name/) { next }
					if ($data{$d} eq $tdata{$d}) { $gotmatch{$templates{$t}[0]} += 1 }
				}
			}	
			my $match = undef;
			my $best = 0;
			foreach my $key (keys %gotmatch) {
				if ($gotmatch{$key} > $best) { $match = $key; $best = $gotmatch{$key} }
			}
			if ($gotmatch{$match} >= $props) { $data{'use'} = $match }
			if (!$data{'use'}) {
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /host_name/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}	
				$data .= $xml_end;			
				my $name = undef;
				if ($data{'icon_image_alt'}) {
					$name = lc("$data{'icon_image_alt'}-$index");
					$name =~ s/_|\s/-/g;
				} elsif ($data{'icon_image'}) {
					$name = lc("$data{'icon_image'}-$index");
					$name =~ s/_|\s|\./-/g;
				} elsif ($data{'vrml_image'}) {
					$name = lc("$data{'vrml_image'}-$index");
					$name =~ s/_|\s|\./-/g;
				} elsif ($data{'statusmap_image'}) {
					$name = lc("$data{'statusmap_image'}-$index");
					$name =~ s/_|\s|\./-/g;
				} else {
					$name = "generic-hostextinfo-$index";
				}
				my $comment = undef;
				my @values = ('',$name,$data,'',$comment);
				my $id = insert_obj_id('extended_host_info_templates',\@values,'hostextinfo_id');
				$data{'use'} = $id;
				$index++;
			}
		} else {
			my %t = fetch_one('extended_host_info_templates','name',$data{'use'});
			$data{'use'} = $t{'hostextinfo_id'};
			if ($data{'2dcoords'} == $t{'2dcoords'}) { delete $data{'2dcoords'} }
			if ($data{'3dcoords'} == $t{'3dcoords'}) { delete $data{'3dcoords'} }
		}
		truncate_table('extended_info_coords');
		foreach my $host (@hosts) {
			my %w = ('name' => $host);
			my %u = ('hostextinfo_id' => $data{'use'});
			update_obj_where('hosts',\%u,\%w);
			if ($data{'2dcoords'} || $data{'3dcoords'}) {
				my %id = fetch_one('hosts','name',$host);
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /coords/) {
						$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my @vals = ($id{'host_id'},$data);
				insert_obj('extended_info_coords',\@vals);	
			}
		}
	}
	delete_one_where('stage_other',\%where);

# parent directive
	update_status('info',"Moving parent directive from templates to hosts");
	$type = 'parent_directive';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @parents = split(/,/, $data{'parents'});	
		my %t = fetch_one('host_templates','name',$objects{$key}[0]);
		my %w = ('hosttemplate_id' => $t{'hosttemplate_id'});
		my @hosts = fetch_list_where('hosts','host_id',\%w);
		foreach my $h (@hosts) {
			$h =~ s/^\s+|\s+$//;
			foreach my $p (@parents) {
				$p =~ s/^\s+|\s+$//;
				my %p = fetch_one('hosts','name',$p);
				my @vals = ($h,$p{'host_id'});
				insert_obj('host_parent',\@vals);
			}
		}
	}
	delete_one_where('stage_other',\%where);

# redundant templates
	update_status('info',"Removing redundant templates");
	$type = 'redundant_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %tnew = fetch_one('host_templates','name',$objects{$key}[2]);
		my %told = fetch_one('host_templates','name',$objects{$key}[0]);
		my %w = ('hosttemplate_id' => $told{'hosttemplate_id'});
		my %u = ('hosttemplate_id' => $tnew{'hosttemplate_id'});
		update_obj_where('hosts',\%u,\%w);
		delete_all('host_templates','name',$objects{$key}[0]);
	}
	delete_one_where('stage_other',\%where);

# hostgroup templates
	update_status('info',"Removing hostgroup templates");
	$type = 'hostgroup_templates';
	my %hostgroup_template = ();
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		$hostgroup_template{$objects{$key}[0]} = $data{'contact_groups'};
	}
	delete_one_where('stage_other',\%where);

# hostgroups
	update_status('info',"Loading hostgroups");
	truncate_table('hostgroup_host');
	$type = 'hostgroup-host';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @contactgroups = split(/,/, $data{'contact_groups'});
		if ($data{'use'}) {
			my @cguse = split(/,/, $hostgroup_template{$data{'use'}});
			push (@contactgroups,@cguse);
		}
		my $id = undef;
		my %obj = fetch_one('hostgroups','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			$id = $obj{'hostgroup_id'};
			my %values = ();
			$values{'alias'} = $objects{$key}[2];
			$values{'comment'} = $objects{$key}[4];
			update_obj('hostgroups','name',$objects{$key}[0],\%values);
		} else {
			my @vals = ('',$objects{$key}[0],$objects{$key}[2],'','','','1',$objects{$key}[4]);
			$id = insert_obj_id('hostgroups',\@vals,'hostgroup_id');
		}
		my %w = ('type' => 'hostgroups','object' => $id);
		delete_one_where('contactgroup_assign',\%w);
		foreach my $cg (@contactgroups) {
			$cg =~ s/^\s+|\s+$//;
			my %c = fetch_one('contactgroups','name',$cg);
			my @vals = ($c{'contactgroup_id'},'hostgroups',$id);
			insert_obj('contactgroup_assign',\@vals);
		}
		if ($data{'members'} eq '*') {
			my @members = fetch_list('hosts','host_id');
			foreach my $mem (@members) {
				my @vals = ($id,$mem);
				insert_obj('hostgroup_host',\@vals);
			}	
		} else {
			my %members = ();
			$data{'members'} =~ s/\s+//g;
			my @mems = split(/,/, $data{'members'});
			foreach (@mems) { $members{$_} = 1 }
			foreach my $mem (keys %members) {
				my %h = fetch_one('hosts','name',$mem);
				if ($h{'host_id'}) {
					my @vals = ($id,$h{'host_id'});
					insert_obj('hostgroup_host',\@vals);
				} else {
					if ($debug) { print "\nError: $mem is not a valid host so will be ignored.\n" }
					update_status('info',"$mem is not a valid host so will be ignored.");
				}
			}	
		}
	}
	delete_one_where('stage_other',\%where);
	return @messages;
}

# services
sub process_services(){
	update_status('info',"Creating service names");
	my $index = 1;
	my $type = 'service';
	my %where = ('type' => $type);
	my %objects = fetch_list_hash_array('stage_other',\%where);
	my %sn_use = ();
	my %sn_cmd = ();
	my %sn_cmdline = ();
	my %sn_id = ();
	my %sn_dep = ();
	foreach my $key (sort keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my %cmd = fetch_one('commands','name',$data{'check_command'});
		$sn_cmd{$objects{$key}[0]} = $cmd{'command_id'};
		$sn_cmdline{$objects{$key}[0]} = $data{'command_line'};
		my $props = 0;
		delete $data{'host_name'};
		delete $data{'hostgroup_name'};
		delete $data{'file_name'};
		my @cgs = split(/,/, $data{'contact_groups'});
		unless ($data{'use'}) {
			my $bestmatch = 0;
			my @contactgroups = ();
			my %w = ();
			my @templates = fetch_list('service_templates','servicetemplate_id');
			if ($data{'check_period'}) {
				my %n = fetch_one('time_periods','name',$data{'check_period'});
				if ($n{'timeperiod_id'}) {
					$data{'check_period'} = $n{'timeperiod_id'}
				} else {
					my $tname = $data{'check_period'};
					update_status('error', "Time period $data{'check_period'} is specified for sercvice $objects{$key}[0] but no such time period exists.");
					my @values = ('',$data{'check_period'},'',$xml_begin.$xml_end,'');
					$data{'check_period'} = insert_obj_id('time_periods',\@values,'timeperiod_id');
					update_status('action', "Added time period $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
				}
			}
			if ($data{'notification_period'}) {
				my %n = fetch_one('time_periods','name',$data{'notification_period'});
				if ($n{'timeperiod_id'}) {
					$data{'notification_period'} = $n{'timeperiod_id'}
				} else {
					my $tname = $data{'notification_period'};
					update_status('error', "Time period $data{'notification_period'} is specified for sercvice $objects{$key}[0] but no such time period exists.");
					my @values = ('',$data{'check_period'},'',$xml_begin.$xml_end,'');
					$data{'notification_period'} = insert_obj_id('time_periods',\@values,'timeperiod_id');
					update_status('action', "Added time period $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
				}
			}
			if ($data{'check_command'}) {
				my %c = fetch_one('commands','name',$data{'check_command'});
				if ($c{'command_id'}) {
					$data{'check_command'} = $c{'command_id'}
				} else {
					my $tname = $data{'check_command'};
					update_status('error', "Check command $data{'check_command'} is specified for sercvice $objects{$key}[0] but no such command definition exists.");
					my @values = ('',$data{'check_period'},'',$xml_begin.$xml_end,'');
					$data{'check_command'} = insert_obj_id('commands',\@values,'command_id');
					update_status('action', "Added check command $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
				}
			}
			my @cgs = split(/,/, $data{'contact_groups'});
			unless (@templates) {
				my $tname = 'default-service';
				update_status('note', "No service templates exist and at least one is required.");
				my $data = $xml_begin.$xml_end;
				my @values = ('',$tname,'','','','','',$data,'');
				$data{'use'} = insert_obj_id('service_templates',\@values,'servicetemplate_id');
				update_status('action', "Added service template $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
			} else {
				foreach my $tid (@templates) {
					my $template_id = $tid;
					my %template = ();
					my $gotmatch = 0;
					my @template_contactgroup = ();
					my $got_parent = 0;
					my $got_contactgroup = 0;
					my $i = 0;
					until ($got_parent || $i == 10) {
						$i++;
						my %tpl = fetch_one('service_templates','servicetemplate_id',$template_id);
						foreach my $t (keys %tpl) {
							unless (exists $template{$t}) { $template{$t} = $tpl{$t} }
						}
						unless ($got_contactgroup) {
							my %where = ('type' => 'service_templates','object' => $template_id);
							@template_contactgroup = fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
							if (@template_contactgroup) { $got_contactgroup = $template_id }
						}
						if ($tpl{'parent_id'}) { 
							$template_id = $tpl{'parent_id'};
						} else {
							$got_parent = 1;
						}
					}
					foreach my $t (keys %template) {
						my $gotmatch = 0;
						foreach my $d (keys %data) {
							if ($data{$d} eq $template{$d}) { $gotmatch++ }
						}	
						my %w = ('type' => 'service_templates','object' => $got_contactgroup);
						my @tcg = fetch_list_where('contactgroup_assign','contactgroup_id',\%w);
						foreach my $cg (@cgs) {
							$cg =~ s/^\s+|\s+$//;
							my %cg = fetch_one('contactgroups','name',$cg);
							push @contactgroups, $cg{'contactgroup_id'};
							foreach (@tcg) { if ($_ eq $cg{'contactgroup_id'}) { $gotmatch++ } }
						}
						if ($gotmatch > $bestmatch) {
							$bestmatch = $gotmatch;
							$data{'use'} = $tid;
						}
					}
				
				}
			}
		} else {
			my %use = fetch_one('service_templates','name',$data{'use'}); 
			if ($use{'servicetemplate_id'}) {
				$data{'use'} = $use{'servicetemplate_id'};
			} else {
				my $tname = $data{'use'};
				update_status('error', "Service $objects{$key}[0] specifies template $tname but no such template exists.");
				my $data = $xml_begin.$xml_end;
				my @values = ('',$tname,'','','','','',$data,'');
				$data{'use'} = insert_obj_id('service_templates',\@values,'servicetemplate_id');
				update_status('action', "Added service template $tname but properties have not been set. WARNING: You must set the properties before attempting preflight or commit!");
			}
		}

		$sn_use{$objects{$key}[0]} = $data{'use'};
		my %sname = fetch_one('service_names','name',$objects{$key}[0]);
		my $id = $sname{'servicename_id'};
		if (!$id) {
			my @values = ('',$objects{$key}[0],$objects{$key}[0],$sn_use{$objects{$key}[0]},$sn_cmd{$objects{$key}[0]},$sn_cmdline{$objects{$key}[0]},'','','');
			$id = insert_obj_id('service_names',\@values,'servicename_id');
		} else {
			my %w = ('servicename_id' => $id);
			my %u = ('template' => $sn_use{$objects{$key}[0]},'check_command' =>  $sn_cmd{$objects{$key}[0]},'command_line' => $sn_cmdline{$objects{$key}[0]});
			update_obj_where('service_names',\%u,\%w);
		}
		$sn_id{$objects{$key}[0]} = $id;
	}
# services
	update_status('info',"Loading services");
	my %service_groups = ();
	my $sqlstmt = "select * from stage_other where type = 'service'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) {
		my %data = parse_xml($values[3]);
		my %hosts = ();
		if ($data{'host_name'} eq '*') {
			my @hosts = fetch_list('hosts','host_id');
			foreach (@hosts) { $hosts{$_} = 1;}
		} elsif ($data{'hostgroup_name'}) {
			my @hostgroups = split(/,/, $data{'hostgroup_name'});
			foreach my $hg (@hostgroups) {
				$hg =~ s/^\s+|\s+$//;
				my %hgid = fetch_one('hostgroups','name',$hg);
				my %w = ('hostgroup_id' => $hgid{'hostgroup_id'});
				my @hosts = fetch_list_where('hostgroup_host','host_id',\%w);
				foreach (@hosts) { $hosts{$_} = 1 }
			}
		} else {
			my @hostnames = split(/,/, $data{'host_name'});
			foreach my $hn (@hostnames) {
				$hn =~ s/^\s+|\s+$//;
				my %h = fetch_one('hosts','name',$hn);
				$hosts{$h{'host_id'}} = 1
			}
		}
		foreach my $hostid (keys %hosts) {
			my $template_id = $sn_use{$values[0]};
			if ($data{'use'}) {
				my %use = fetch_one('service_templates','name',$data{'use'});
				if ($use{'servicetemplate_id'}) {
					$template_id = $use{'servicetemplate_id'};
				} else {
					my %t = fetch_one('service_templates','servicetemplate_id',$template_id);
					update_status('error',"Service template $data{'use'} for service $values[0] does not exist. Template $t{'name'} assigned.");

				}
			}
			my $id = undef;
			my %w = ('host_id' => $hostid,'servicename_id' => $sn_id{$values[0]});
			my %obj = fetch_one_where('services',\%w);
			if ($obj{'service_id'}) {
				$id = $obj{'service_id'};
				my %values = ();
				$values{'servicename_id'} = $sn_id{$values[0]};
				$values{'servicetemplate_id'} = $template_id;
				$values{'check_command'} = $sn_cmd{$values[0]};
				$values{'command_line'} = $data{'command_line'};
				$values{'comment'} = $values[4];
				update_obj('services','service_id',$id,\%values);
			} else {
				my @vals = ('',$hostid,$sn_id{$values[0]},$template_id,'','','1',$sn_cmd{$values[0]},$data{'command_line'},$values[4]);
				$id = insert_obj_id('services',\@vals,'service_id');
			}
# Service overrides
			my @template_contactgroup = ();
			my %template = ();
			my $got_parent = 0;
			my $got_contactgroup = 0;
			my $i = 0;
			until ($got_parent || $i == 10) {
				$i++;
				my %tpl = fetch_one('service_templates','servicetemplate_id',$template_id);
				foreach my $t (keys %tpl) {
					unless (exists $template{$t}) { $template{$t} = $tpl{$t} }
				}
				unless ($got_contactgroup) {
					my %where = ('type' => 'service_templates','object' => $template_id);
					@template_contactgroup = fetch_list_where('contactgroup_assign','contactgroup_id',\%where);
					if (@template_contactgroup) { $got_contactgroup = 1 }
				}
				if ($tpl{'parent_id'}) { 
					$template_id = $tpl{'parent_id'};
				} else {
					$got_parent = 1;
				}
			}
			my %t = fetch_one('time_periods','name',$data{'check_period'});
			my %n = fetch_one('time_periods','name',$data{'notification_period'});
			my %e = fetch_one('commands','name',$data{'event_handler'});
			if ($template{'check_period'} eq $t{'timeperiod_id'}) { delete $t{'timeperiod_id'} }
			if ($template{'notification_period'} eq $n{'timeperiod_id'}) { delete $n{'timeperiod_id'} }
			if ($template{'event_handler'} eq $e{'command_id'}) { delete $e{'command_id'} }
			delete $data{'check_period'};
			delete $data{'notification_period'};
			delete $data{'event_handler'};
			delete $data{'use'};
			delete $data{'host_name'};
			delete $data{'check_command'};
			delete $data{'command_line'};
			my $data = undef; 
			foreach my $d (keys %data) {
				unless ($d =~ /contact_groups|file_name|service_groups/ || $data{$d} eq $template{$d}) { 
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}
			}	
			delete_all('service_overrides','service_id',$id);
			if ($t{'timeperiod_id'} || $n{'timeperiod_id'} || $e{'command_id'} || $data) {
				$data = $xml_begin.$data.$xml_end;
				my @values = ($id,$t{'timeperiod_id'},$n{'timeperiod_id'},$e{'command_id'},$data);
				my $result = insert_obj('service_overrides',\@values);
			}
			my @tcn = ();
			foreach my $mem (@template_contactgroup) {
				my %c = fetch_one('contactgroups','contactgroup_id',$mem);
				push @tcn, $c{'name'};
			}
			@tcn = sort @tcn;
			my $tstr = undef;
			foreach my $mem (@tcn) {
				$tstr .= $mem;
			}
			
			my %w = ('type' => 'services','object' => $id);
			delete_one_where('contactgroup_assign',\%w);
			my @members = split(/,/, $data{'contact_groups'});
			@members = sort @members;
			my $memstr = undef;
			foreach my $mem (@members) {
				$mem =~ s/^\s+|\s+$//;
				$memstr .= $mem;
			}
			unless ($tstr eq $memstr) {
				foreach my $mem (@members) {
					$mem =~ s/^\s+|\s+$//;
					my %c = fetch_one('contactgroups','name',$mem);
					my @vals = ($c{'contactgroup_id'},'services',$id);
					insert_obj('contactgroup_assign',\@vals);
				}
			}		
			delete_all('file_service','service_id',$id);
			if ($data{'service_groups'}) {	$service_groups{"$hostid-$id"} = $data{'service_groups'} }	
			my %f = fetch_one('files','name',$data{'file_name'});
			my @vals = ($f{'file_id'},$id);
			insert_obj('file_service',\@vals);
			
		}		
	}
	$sth->finish;
	delete_one_where('stage_other',\%where);
	
# servicegroups
	update_status('info',"Loading servicegroups");
	truncate_table('servicegroup_service');
	$type = 'servicegroup';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @members = split(/,/, $data{'members'});
		my $id = undef;
		my %obj = fetch_one('servicegroups','name',$objects{$key}[0]);
		if ($obj{'name'}) {
			$id = $obj{'servicegroup_id'};
			my %values = ();
			$values{'alias'} = $objects{$key}[2];
			update_obj('servicegroups','name',$objects{$key}[0],\%values);
		} else {
			my @vals = ('',$objects{$key}[0],$objects{$key}[2]);
			$id = insert_obj_id('servicegroups',\@vals,'servicegroup_id');
		}
		my @services = ();
		my @hosts = ();
		my $pairs = 0;
		my $i = 0;
		foreach my $m (@members) {
			if ($i == 1) {
				push @services, $m;	$i = 0;
			} else {
				push @hosts, $m; $i = 1; $pairs++;
			}
		}

		for ($i = 0; $i < $pairs; $i++) {
			my %h =  fetch_one('hosts','name',$hosts[$i]);
			my %sn =  fetch_one('service_names','name',$services[$i]);
			my %where = ('host_id' => $h{'host_id'},'servicename_id' => $sn{'servicename_id'});
			my %s = fetch_one_where('services',\%where);
			my @vals = ($id,$h{'host_id'},$s{'service_id'});
			insert_obj('servicegroup_service',\@vals);
		}
	}
	delete_one_where('stage_other',\%where);

	foreach my $hs (keys %service_groups) {
		my @sh = split(/,/, $hs);
		my %sg = fetch_one('servicegroups','name',$service_groups{$hs});
		my @vals = ($sg{'servicegroup_id'},$sh[0],$sh[1]);
		insert_obj('servicegroup_service',\@vals);	
	}

# service dependency templates
	update_status('info',"Loading service dependency templates");
	$type = 'servicedependency_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		if ($data{'service_description'}) {
			my %s = fetch_one('service_names','name',$data{'service_description'});
			if (!$s{'servicename_id'}) {							
				my @values = ('',$data{'service_description'},$data{'service_description'},'0','0','0','','','');
				my $id = insert_obj_id('service_names',\@values,'servicename_id');
				$s{'servicename_id'} = $id;
				$sn_id{$data{'service_description'}} = $id;
				update_status('note', "Service name $data{'service_description'} does not exist and is required for dependency template $objects{$key}[0].");
				update_status('action', "Service name $data{'service_description'} added for dependency template $objects{$key}[0]. Advise you check properties.");
			}
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /notification|execution/) {
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}
			}	
			$data .= $xml_end;
			my %obj = fetch_one('service_dependency_templates','name',$objects{$key}[0]);
			if ($obj{'name'}) {
				my %values = ();
				$values{'data'} = $data;
				update_obj('service_dependency_templates','name',$objects{$key}[0],\%values);
			} else {
				my @vals = ('',$objects{$key}[0],$s{'servicename_id'},$data,$objects{$key}[4]);
				insert_obj('service_dependency_templates',\@vals);
			}
		} else {
			my $message = ();
			foreach my $name (keys %data) {
				$message .= "$name: $data{$name}<br>";
			}
			update_status('error',"Unable to create service dependency template: No service description.<br><br>$message");
		}
	}
	delete_one_where('stage_other',\%where);
	

# service dependency
	update_status('info',"Loading service dependencies");
	truncate_table('service_dependency');
	$type = 'servicedependency';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	$index = count('service_dependency_templates');
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my $props = 0;
		if ($data{'service_description'}) { $props++ }
		if ($data{'execution_failure_criteria'}) { $props++ }
		if ($data{'notification_failure_criteria'}) { $props++ }
		if ($data{'use'}) {
			my %t = fetch_one('service_dependency_templates','name',$data{'use'});
			$data{'use'} = $t{'id'};
		}
		if (!$data{'use'}) {
			my %w = ();
			my %templates = fetch_list_hash_array('service_dependency_templates',\%w);
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[3]);
				my $gotmatch = 0;
				my %s = fetch_one('service_names','servicename_id',$templates{$t}[2]);
				if ($data{'service_description'} eq $s{'name'}) { $gotmatch++ }
				if ($data{'execution_failure_criteria'} eq $tdata{'execution_failure_criteria'}) { $gotmatch++ }
				if ($data{'notification_failure_criteria'} eq $tdata{'notification_failure_criteria'}) { $gotmatch++ }
				if ($gotmatch == $props) {
					$data{'use'} = $templates{$t}[0];
					last;
				}
			}
		}

		if (!$data{'use'} && $props > 0) {
			my %s = fetch_one('service_names','name',$data{'service_description'});
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /^service_description$|dependent|host_name/) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $data{'service_description'};
			$name =~ s/_/-/g;
			$name = "dep-$name-$index";
			%s = fetch_one('service_names','name',$data{'service_description'});
			my @vals = ('',$name,$s{'servicename_id'},$data,$objects{$key}[4]);
			$data{'use'} = insert_obj_id('service_dependency_templates',\@vals,'id');
		}

		my @dhn = split(/,/, $data{'dependent_host_name'});
		my @dependent_hosts = ();
		foreach my $dhn (@dhn) {
			$dhn =~ s/^\s+|\s+$//;
			my %d = fetch_one('hosts','name',$dhn);
			push @dependent_hosts, $d{'host_id'};
		}
		my @phn = split(/,/, $data{'host_name'});
		my @parent_hosts = ();
		foreach my $phn (@phn) {
			$phn =~ s/^\s+|\s+$//;
			my %h = fetch_one('hosts','name',$phn);
			push @parent_hosts, $h{'host_id'};
		}
		@dhn = split(/,/, $data{'dependent_hostgroup_name'});
		foreach my $dhn (@dhn) {
			$dhn =~ s/^\s+|\s+$//;
			my %d = fetch_one('hostgroups','name',$dhn);
			my %w = ('hostgroup_id' => $d{'hostgroup_id'});
			my @hids = fetch_list_where('hostgroup_host','host_id',\%w);
			push (@dependent_hosts, @hids);
		}
		@phn = split(/,/, $data{'hostgroup_name'});
		foreach my $phn (@phn) {
			$phn =~ s/^\s+|\s+$//;
			my %h = fetch_one('hostgroups','name',$phn);
			my %w = ('hostgroup_id' => $h{'hostgroup_id'});
			my @hids = fetch_list_where('hostgroup_host','host_id',\%w);
			push (@parent_hosts, @hids);
		}

		foreach my $dhn (@dependent_hosts) {
			foreach my $phn (@parent_hosts) {
				$objects{$key}[0] =~ s/-\d+$//;
				my $sqlstmt = "select service_id from services where host_id = '$dhn' and servicename_id in "
				. "(select servicename_id from service_names where name = '$objects{$key}[0]')";	
				my $sid = $dbh->selectrow_array($sqlstmt);
				if ($sid) {	
					if ($phn eq $dhn) { $sn_dep{$objects{$key}[0]} = $data{'use'} };
					my @vals = ('',$sid,$dhn,$phn,$data{'use'},$objects{$key}[4]);
					insert_obj('service_dependency',\@vals);
				} else {
					update_status('error', "service $objects{$key}[0] does not exist for dependency $sqlstmt");
				}
			}
		}
	}
	delete_one_where('stage_other',\%where);
	
# Update Service names
	update_status('info',"Updating service names with dependency info");
	foreach my $dep (keys %sn_dep) {
		my %u = ('dependency' => $sn_dep{$dep});
		my %w = ('name' => $dep);
		update_obj_where('service_names',\%u,\%w);
	}

# Service extended info
	update_status('info',"Loading service extended info");
	my %sn_extinfo = ();
	$type = 'serviceextinfo';
	%where = ('type' => $type);
	my %service_desc = ();
	%objects = fetch_list_hash_array('stage_other',\%where);
	$index = count('extended_service_info_templates');
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		if ($data{'notes_url'}) { 
			$data{'notes_url'} =~ s/$objects{$key}[2]/\$HOSTNAME\$/g;
			$data{'notes_url'} =~ s/$objects{$key}[0]/\$SERVICENAME\$/g;
			$data{'notes_url'} =~ s/\s/+/g;
		}
		if (!$data{'use'}) {
			my $props = 0;
			foreach (keys %data) { $props++ }
			if ($data{'host_name'}) { $props -= 1 }
			if ($data{'service_description'}) { $props -= 1 }
			my %w = ();
			my %templates = fetch_list_hash_array('extended_service_info_templates',\%w);
			my %gotmatch = ();
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[2]);
				foreach my $d (keys %data) {
					if ($data{$d} eq $tdata{$d}) { $gotmatch{$templates{$t}[0]} += 1 }
				}	
			}
			my $match = undef;
			my $best = 0;
			foreach my $key (keys %gotmatch) {
				if ($gotmatch{$key} > $best) { $match = $key; $best = $gotmatch{$key} }
			}
			if ($gotmatch{$match} >= $props) { $data{'use'} = $match	}
			$sn_extinfo{$objects{$key}[0]} = $data{'use'};
			if (!$data{'use'}) {
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /host_name/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}	
				$data .= $xml_end;
				my $name = undef;
				if (!$service_desc{$objects{$key}[0]}) {
					$name = lc("$objects{$key}[0]-$index");
					$name =~ s/_|\s/-/g;
					$service_desc{$objects{$key}[0]} = 1;
				} elsif ($data{'icon_image_alt'}) {
					$name = lc("$data{'icon_image_alt'}-$index");
					$name =~ s/_|\s/-/g;
				} elsif ($data{'icon_image'}) {
					$name = lc("$data{'icon_image'}-$index");
					$name =~ s/_|\s|\./-/g;
				} else {
					$name = "generic-serviceextinfo-$index";
				}
				my $comment = undef;
				my @values = ('',$name,$data,'',$comment);
				my $id = insert_obj_id('extended_service_info_templates',\@values,'serviceextinfo_id');
				if ($id =~ /Error/) { update_status('error',$id) }
				$data{'use'} = $name;
				$sn_extinfo{$objects{$key}[0]} = $id;
			}
		} else {
			my %t = fetch_one('extended_service_info_templates','name',$data{'use'});
			$sn_extinfo{$objects{$key}[0]} = $t{'serviceextinfo_id'};
		}
		if ($data{'notes_url'}) {
			my %t = fetch_one('extended_service_info_templates','name',$data{'use'});
			delete $t{'script'};
			delete $t{'name'};
			delete $t{'comment'};
			delete $t{'serviceextinfo_id'};
			$t{'notes_url'} = $data{'notes_url'};
			my $data = $xml_begin;
			foreach my $d (keys %t) {
				$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my %w = ('name' => $data{'use'});
			my %u = ('data' => $data);
			update_obj_where('extended_service_info_templates',\%u,\%w);
			
		}
	}
	delete_one_where('stage_other',\%where);
	

	foreach my $service (keys %sn_extinfo) {
		my %w = ('name' => $service);
		my %u = ('extinfo' => $sn_extinfo{$service});
		update_obj_where('service_names',\%u,\%w);
		
		my %snid = fetch_one('service_names','name',$service);
		%w = ('servicename_id' => $snid{'servicename_id'});
		%u = ('serviceextinfo_id' => $sn_extinfo{$service});
		update_obj_where('services',\%u,\%w);
		
	}
	return @messages;
}


########################################################################################################
# escalations
############################################################################
# 

sub process_escalations() {
	my @escalation_tables = ('escalation_tree_template','escalation_trees','tree_template_contactgroup');
	foreach (@escalation_tables) { truncate_table($_) }

# hostgroup escalations
	update_status('info',"Loading hostgroup escalations");
	my %splat = fetch_one('service_names','name','*');
	my $splat_id = $splat{'servicename_id'};
	my $sqlstmt = "truncate table stage_escalations";
	my $sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('error', "Error: $sqlstmt $@") if $@;
	my $index = 0;
	my $type = 'hostgroupescalation';
	my %where = ('type' => $type);
	my %objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objs = split(/,/, $data{'hostgroup_name'});
		my @objids;
		foreach my $obj (@objs) {
			$obj =~ s/^\s+|\s+$//;
			my %o = fetch_one('hostgroups','name',$obj);
			push @objids, $o{'hostgroup_id'};
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my @values = ('',"$name",'hostgroup',$data,$objects{$key}[4],$splat_id);
			$data{'use'} = insert_obj_id('escalation_templates',\@values,'template_id');
		} else {	
			my %w = ('type' => 'hostgroup','name' => $data{'use'});
			my %t = fetch_one_where('escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my $tname = $objects{$key}[0];
				$tname =~ s/-\d+$//;
				my @values = ('',"$tname-$index",'hostgroup',$data,$objects{$key}[4],$splat_id);
				my $id = insert_obj_id('escalation_templates',\@values,'template_id');
				if ($id =~ /Error/) { update_status('error',$id) }
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'hostgroup_escalation_templates',$id);
					insert_obj('contactgroup_assign',\@vals);
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		insert_obj('stage_escalations',\@values);
	}
	my @trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hostgroups = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('hostgroups','hostgroup_id',$hostgroups[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'hostgroup');
		my $tree_id = insert_obj_id('escalation_trees',\@vals,'tree_id');
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			insert_obj('escalation_tree_template',\@vals);
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				insert_obj('tree_template_contactgroup',\@cgvals);
			}
		}
		foreach my $hg (@hostgroups) {
			$hg =~ s/^\s+|\s+$//;
			my %w = ('hostgroup_id' => $hg);
			my %u = ('hostgroup_escalation_id' => $tree_id);
			update_obj_where('hostgroups',\%u,\%w);
		}
	}
	delete_one_where('stage_other',\%where);

########################################################################################################
# host escalations
############################################################################
# host escalation - host
	update_status('info',"Loading host escalations");
	update_status('info',"--hostgroups");
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('error', "Error: $sqlstmt $@") if $@;
	$type = 'hostescalation';
	%where = ('type' => $type, 'parent' => 'hostgroup');
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objs = split(/,/, $data{'hostgroup_name'});
		my @objids = ();
		foreach my $obj (@objs) {
			$obj =~ s/^\s+|\s+$//;
			my %o = fetch_one('hostgroups','name',$obj);
			push @objids, $o{'hostgroup_id'};
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my @values = ('',"$name",'host',$data,$objects{$key}[4],$splat_id);
			$data{'use'} = insert_obj_id('escalation_templates',\@values,'template_id');
		} else {	
			my %w = ('type' => 'host','name' => $data{'use'});
			my %t = fetch_one_where('escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host/) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my $tname = $objects{$key}[0];
				$tname =~ s/-\d+$//;
				my @values = ('',"$tname-$index",'host',$data,$objects{$key}[4],$splat_id);
				my $id = insert_obj_id('escalation_templates',\@values,'template_id');
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'host_escalation_templates',$id);
					insert_obj('contactgroup_assign',\@vals);
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		insert_obj('stage_escalations',\@values);
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hostgroups = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('hostgroups','hostgroup_id',$hostgroups[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'host');
		my $tree_id = insert_obj_id('escalation_trees',\@vals,'tree_id');
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			insert_obj('escalation_tree_template',\@vals);
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				insert_obj('tree_template_contactgroup',\@cgvals);
			}
		}
		foreach my $hg (@hostgroups) {
			$hg =~ s/^\s+|\s+$//;
			my %w = ('hostgroup_id' => $hg);
			my %u = ('host_escalation_id' => $tree_id);
			update_obj_where('hostgroups',\%u,\%w);
		}
	}
	delete_one_where('stage_other',\%where);

############################################################################
# host escalation - host
	update_status('info',"--hosts");
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('error', "Error: $sqlstmt $@") if $@;
	$type = 'hostescalation';
	%where = ('type' => $type, 'parent' => 'host');
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objids = ();
		if ($data{'host_name'} eq '*') {
			@objids = fetch_list('hosts','host_id');
		} else {
			my @objs = split(/,/, $data{'host_name'});
			foreach my $obj (@objs) {
				$obj =~ s/^\s+|\s+$//;
				my %o = fetch_one('hosts','name',$obj);
				push @objids, $o{'host_id'};
			}
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my @values = ('',"$name",'host',$data,$objects{$key}[4],$splat_id);
			$data{'use'} = insert_obj_id('escalation_templates',\@values,'template_id');
		} else {	
			my %w = ('type' => 'host','name' => $data{'use'});
			my %t = fetch_one_where('escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my $tname = $objects{$key}[0];
				$tname =~ s/-\d+$//;
				my @values = ('',"$tname-$index",'host',$data,$objects{$key}[4],$splat_id);
				my $id = insert_obj_id('escalation_templates',\@values,'template_id');
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'host_escalation_templates',$id);
					insert_obj('contactgroup_assign',\@vals);
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		insert_obj('stage_escalations',\@values);
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hosts = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('hosts','host_id',$hosts[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'host');
		my $tree_id = insert_obj_id('escalation_trees',\@vals,'tree_id');
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			insert_obj('escalation_tree_template',\@vals);
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				insert_obj('tree_template_contactgroup',\@cgvals);
			}
		}
		foreach my $h (@hosts) {
			$h =~ s/^\s+|\s+$//;
			my %w = ('host_id' => $h);
			my %u = ('host_escalation_id' => $tree_id);
			update_obj_where('hosts',\%u,\%w);
		}
	}
	delete_one_where('stage_other',\%where);

########################################################################################################
# service escalations
############################################################################
# service escalation - hostgroup
	update_status('info',"Loading service escalations");
	update_status('info',"--hostgroups");
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('error', "Error: $sqlstmt $@") if $@;
	$type = 'serviceescalation';
	%where = ('type' => $type, 'parent' => 'hostgroup');
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objs = split(/,/, $data{'hostgroup_name'});
		my @objids = ();
		foreach my $obj (@objs) {
			$obj =~ s/^\s+|\s+$//;
			my %o = fetch_one('hostgroups','name',$obj);
			push @objids, $o{'hostgroup_id'};
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				if ($d eq 'service_description') {
					my %s = fetch_one('service_names','name',$data{$d});
					$objects{$d} = $s{'servicename_id'};
				} else {
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my %sn = fetch_one('service_names','name',$objects{'service_description'});
			my @values = ('',"$name",'service',$data,$objects{$key}[4],$sn{'servicename_id'});
			$data{'use'} = insert_obj_id('escalation_templates',\@values,'template_id');
		} else {	
			my %w = ('type' => 'service','name' => $data{'use'});
			my %t = fetch_one_where('escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} elsif ($d eq 'service_description') {
						my %s = fetch_one('service_names','name',$data{$d});
						$objects{$d} = $s{'servicename_id'};
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my %sn = fetch_one('service_names','name',$objects{'service_description'});
				my $tname = $objects{$key}[0];
				$tname =~ s/-\d+$//;
				my @values = ('',"$tname-$index",'service',$data,$objects{$key}[4],$sn{'servicename_id'});
				my $id = insert_obj_id('escalation_templates',\@values,'template_id');
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'service_escalation_templates',$id);
					insert_obj('contactgroup_assign',\@vals);
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		insert_obj('stage_escalations',\@values);
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hostgroups = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('hostgroups','hostgroup_id',$hostgroups[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'service');
		my $tree_id = insert_obj_id('escalation_trees',\@vals,'tree_id');
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			insert_obj('escalation_tree_template',\@vals);
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				insert_obj('tree_template_contactgroup',\@cgvals);
			}
		}
		foreach my $hg (@hostgroups) {
			$hg =~ s/^\s+|\s+$//;
			my %w = ('hostgroup_id' => $hg);
			my %u = ('service_escalation_id' => $tree_id);
			update_obj_where('hostgroups',\%u,\%w);
		}
	}
	delete_one_where('stage_other',\%where);

############################################################################
# service escalation - host
	update_status('info',"--hosts");
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('error', "Error: $sqlstmt $@") if $@;
	$type = 'serviceescalation';
	%where = ('type' => $type, 'parent' => 'host');
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objids;
		if ($data{'host_name'} eq '*') {
			@objids = fetch_list('hosts','host_id');
		} else {
			my @objs = split(/,/, $data{'host_name'});
			foreach my $obj (@objs) {
				$obj =~ s/^\s+|\s+$//;
				my %o = fetch_one('hosts','name',$obj);
				push @objids, $o{'host_id'};
			}
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				if ($d eq 'service_description') {
					my %s = fetch_one('service_names','name',$data{$d});
					$objects{$d} = $s{'servicename_id'};
				} else {
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my %sn = fetch_one('service_names','name',$objects{'service_description'});
			my @values = ('',"$name",'service',$data,$objects{$key}[4],$sn{'servicename_id'});
			$data{'use'} = insert_obj_id('escalation_templates',\@values,'template_id');
		} else {	
			my %w = ('type' => 'service','name' => $data{'use'});
			my %t = fetch_one_where('escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} elsif ($d eq 'service_description') {
						my %s = fetch_one('service_names','name',$data{$d});
						$objects{$d} = $s{'servicename_id'};
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my $tname = $objects{$key}[0];
				$tname =~ s/-\d+$//;
				my %sn = fetch_one('service_names','name',$objects{'service_description'});
				my @values = ('',"$tname-$index",'service',$data,$objects{$key}[4],$sn{'servicename_id'});
				my $id = insert_obj_id('escalation_templates',\@values,'template_id');
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'service_escalation_templates',$id);
					insert_obj('contactgroup_assign',\@vals);
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		insert_obj('stage_escalations',\@values);
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hosts = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('hosts','host_id',$hosts[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'service');
		my $tree_id = insert_obj_id('escalation_trees',\@vals,'tree_id');
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			insert_obj('escalation_tree_template',\@vals);
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				insert_obj('tree_template_contactgroup',\@cgvals);
			}
		}
		foreach my $h (@hosts) {
			$h =~ s/^\s+|\s+$//;
			my %w = ('host_id' => $h);
			my %u = ('service_escalation_id' => $tree_id);
			update_obj_where('hosts',\%u,\%w);
		}
	}
############################################################################
# service escalation - service
	update_status('info',"--services");
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('error', "Error: $sqlstmt $@") if $@;
	$type = 'serviceescalation-service';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objids = ();
		if ($data{'host_name'} eq '*') {
			@objids = fetch_list('hosts','host_id');
		} else {
			my @objs = split(/,/, $data{'host_name'});
			foreach my $obj (@objs) {
				$obj =~ s/^\s+|\s+$//;
				my %o = fetch_one('hosts','name',$obj);
				push @objids, $o{'host_id'};
			}
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = "$objects{$key}[0]%%";
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				if ($d eq 'service_description') {
					my %s = fetch_one('service_names','name',$data{$d});
					$objects{$d} = $s{'servicename_id'};
				} else {
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my %sn = fetch_one('service_names','name',$objects{'service_description'});
			my @values = ('',"$name",'service',$data,$objects{$key}[4],$sn{'servicename_id'});
			$data{'use'} = insert_obj_id('escalation_templates',\@values,'template_id');
		} else {	
			my %w = ('type' => 'service','name' => $data{'use'});
			my %t = fetch_one_where('escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} elsif ($d eq 'service_description') {
						my %s = fetch_one('service_names','name',$data{$d});
						$objects{$d} = $s{'servicename_id'};
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;

				my $name = $t{'name'};
				$name =~ s/-\d+$//;
				my %sn = fetch_one('service_names','name',$objects{'service_description'});
				my @values = ('',"$name-$index",'service',$data,$objects{$key}[4],$sn{'servicename_id'});
				my $id = insert_obj_id('escalation_templates',\@values,'template_id');
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'service_escalation_templates',$id);
					insert_obj('contactgroup_assign',\@vals);
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		insert_obj('stage_escalations',\@values);
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @sn = split(/%%/, $obj);
		my @hosts = split(/,/, $sn[1]);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('hosts','host_id',$hosts[0]);
		my $name = "esctree-$sn[0]-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'service');
		my $tree_id = insert_obj_id('escalation_trees',\@vals,'tree_id');
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			insert_obj('escalation_tree_template',\@vals);
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				insert_obj('tree_template_contactgroup',\@cgvals);
			}
		}
		$sn[0] =~ s/-\d+//;
		my %sn = fetch_one('service_names','name',$sn[0]);
		foreach my $hid (@hosts) {
			my %w = ('host_id' => $hid,'servicename_id' => $sn{'servicename_id'});
			my %u = ('escalation_id' => $tree_id);
			update_obj_where('services',\%u,\%w);
		}
	}



	delete_one_where('stage_other',\%where);
	$dt = datetime();
	update_status('completed',"$dt");
	return @messages;
}


if ($debug) {
	my $sqlstmt = "select value from setup where name = 'nagios_etc'";
	my $nagios_etc = $dbh->selectrow_array($sqlstmt);
	truncate_tables();
	stage_load('',$nagios_etc);
	process_objects();
	process_hosts();
	process_services();
	process_escalations();
}

1;

