#!/usr/bin/perl --
# MonArch - Groundwork Monitor Architect
# monarch_setup.pl
#
###############################################################################
# Release 0.97a
# 30-Sept-2005
###############################################################################
#

use DBI;
use strict;
use File::Copy;

my $version = '0.97a';
my ($dsn, $user,$passwd) = 0;
my $drop_tables = $ARGV[0];
my $database = "monarch";
my $databaseuser = "root";
my $databasepasswd = "";
my $databasehost = 'localhost';
my $cgi_bin = "/srv/www/cgi-bin";
my $doc_root = "/srv/www/htdocs";
my $web_group = "www";
my $web_user = "runwww";
my $monarch_home = "/usr/local/groundwork/monarch";
my $nagios_etc = "/etc/nagios";
my $nagios_bin = "/usr/sbin";
my $set_perms = 0;
my $is_portal = 0;
my $nagios_ver = '1.x';
my $error = 0;
if (-e '/etc/redhat-release') {
	$cgi_bin = "/var/www/cgi-bin";
	$doc_root = "/var/www/html";
	$web_group = "apache";
	$web_user = "apache";
}


sub check_prereqs() {
	my @missing_reqs = ();
	my $errors = qx('./check_mods.pl');
	if ($errors) {
		push @missing_reqs, $errors;
		push @missing_reqs, "Missing perl module requirements. See README for list";
		push @missing_reqs, "To check installed modules do:";
		push @missing_reqs, "perl -MFile::Find=find -MFile::Spec::Functions -lwe  'find { wanted => sub { print canonpath $_ if /\.pm\z/ }, no_chdir => 1 }, \@INC'";
	}
	my $ver = qx(mysql -V);
	unless ($ver =~ /Distrib 4.1/i) {
		print "\n$ver";
		print "\nWarning: MySQL ver 4.1x required (ignore if database is remote).\n\n";
	}

	if (@missing_reqs) {
		print "\nPrerequisite check failed (see README.txt). Please correct the following:\n";
		foreach (@missing_reqs) {
			print "\n\t$_\n";
		}
		exit;
	}
}

#
####################################################################################
# Drop Tables
####################################################################################
#
sub drop_tables() {
	my $dsn = "DBI:mysql:$database:$databasehost";
	my $dbh = DBI->connect($dsn, $databaseuser, $databasepasswd, {'RaiseError' => 1});

	# Associative tables
	eval { $dbh->do("DROP TABLE external_host_profile") };
	print "\ndrop external_host_profile failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE profile_hostgroup") };
	print "\ndrop profile_hostgroup failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE profile_parent") };
	print "\ndrop profile_parent failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE profile_host_profile_service") };
	print "\ndrop profile_host_profile_service failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE profile_service_host") };
	print "\ndrop profile_service_host failed: $@\n" if $@;
	
	eval { $dbh->do("DROP TABLE profiles_service") };
	print "\ndrop profiles_service failed: $@\n" if $@;

	# Files
	eval { $dbh->do("DROP TABLE file_host") };
	print "\ndrop file_host failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE file_service") };
	print "\ndrop file_service failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE files") };
	print "\ndrop files failed: $@\n" if $@;

	# Externals
	eval { $dbh->do("DROP TABLE external_host") };
	print "\ndrop external_host failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE external_service") };
	print "\ndrop external_service failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE external_service_names") };
	print "\ndrop external_service_names failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE externals") };
	print "\ndrop externals failed: $@\n" if $@;

	# Time Periods
	eval { $dbh->do("DROP TABLE time_periods") };
	print "\ndrop time_periods failed: $@\n" if $@;

	# contacts
	eval { $dbh->do("DROP TABLE contact_command") };
	print "\ndrop contact_command failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE contactgroup_contact") };
	print "\ndrop contactgroup_contact failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE contactgroup_assign") };
	print "\ndrop contactgroup_assign failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE contact_overrides") };
	print "\ndrop contacts failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE contact_command_overrides") };
	print "\ndrop contact_command_overrides failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE contact_templates") };
	print "\ndrop contacts failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE contactgroups") };
	print "\ndrop contact_groups failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE contacts") };
	print "\ndrop contacts failed: $@\n" if $@;

	# Commands
	eval { $dbh->do("DROP TABLE commands") };
	print "\ndrop check_commands failed: $@\n" if $@;

	# Services
	eval { $dbh->do("DROP TABLE service_overrides") };
	print "\ndrop service_overrides failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE servicegroup_service") };
	print "\ndrop servicegroup_service failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE servicegroups") };
	print "\ndrop servicegroups failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE extended_service_info_templates") };
	print "\ndrop extended_service_info_templates failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE service_dependency") };
	print "\ndrop service_dependency failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE service_dependency_templates") };
	print "\ndrop service_dependency_templates failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE services") };
	print "\ndrop services failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE service_templates") };
	print "\ndrop service_templates failed: $@\n" if $@;

	# Hosts 
	eval { $dbh->do("DROP TABLE host_overrides") };
	print "\ndrop host_overrides failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE extended_host_info_templates") };
	print "\ndrop extended_host_info_templates failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE extended_info_coords") };
	print "\ndrop extended_info_coords failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE hostgroup_host") };
	print "\ndrop hostgroup_host failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE host_dependencies") };
	print "\ndrop host_dependencies failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE host_parent") };
	print "\ndrop host_parent failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE hostgroups") };
	print "\ndrop hostgroups failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE host_templates") };
	print "\ndrop host_templates failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE hosts") };
	print "\ndrop hosts failed: $@\n" if $@;

	# Escalations
	eval { $dbh->do("DROP TABLE escalation_trees") };
	print "\ndrop escalation_trees failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE escalation_templates") };
	print "\ndrop escalation_templates failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE escalation_tree_template") };
	print "\ndrop escalation_tree_template failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE tree_template_contactgroup") };
	print "\ndrop tree_template_contactgroup failed: $@\n" if $@;

	# Profiles

	eval { $dbh->do("DROP TABLE profiles_host") };
	print "\ndrop profiles_host failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE serviceprofile") };
	print "\ndrop serviceprofile failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE service_names") };
	print "\ndrop service_names failed: $@\n" if $@;

	# And the rest...
	eval { $dbh->do("DROP TABLE setup") };
	print "\ndrop setup failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE user_group") };
	print "\ndrop user_group failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE access_list") };
	print "\ndrop access_list failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE users") };
	print "\ndrop users failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE user_groups") };
	print "\ndrop user_groups failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE import_schemas") };
	print "\ndrop import_schemas failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE match_strings") };
	print "\ndrop match_strings failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE stage_hosts") };
	print "\ndrop stage_hosts failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE stage_host_services") };
	print "\ndrop stage_host_services failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE stage_host_hostgroups") };
	print "\ndrop stage_host_hostgroups failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE stage_other") };
	print "\ndrop stage_other failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE stage_status") };
	print "\ndrop stage_status failed: $@\n" if $@;

	eval { $dbh->do("DROP TABLE stage_escalations") };
	print "\ndrop stage_escalations failed: $@\n" if $@;
	if ($dbh) {
	   $dbh and $dbh->disconnect();
	}

}

sub create_db() {
	my $dsn = "DBI:mysql:$database:$databasehost";
	my $dbh = DBI->connect($dsn, $databaseuser, $databasepasswd, {'RaiseError' => 1});

	#
	####################################################################################
	# Create Tables
	####################################################################################
	# Commands
	#

	$dbh->do("CREATE TABLE commands (command_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			type VARCHAR(50),
			data TEXT,
			comment TEXT) TYPE=INNODB");

	#
	####################################################################################
	# contact Templates
	#

	$dbh->do("CREATE TABLE contact_templates (contacttemplate_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			host_notification_period SMALLINT(4) UNSIGNED,
			service_notification_period SMALLINT(4) UNSIGNED,
			data TEXT,
			comment TEXT) TYPE=INNODB");

	#
	####################################################################################
	# contact commands
	#

	$dbh->do("CREATE TABLE contact_command (contacttemplate_id SMALLINT(4) UNSIGNED,
			type VARCHAR(50),
			command_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (contacttemplate_id,type,command_id),
			FOREIGN KEY (command_id) REFERENCES commands(command_id) ON DELETE CASCADE,
			FOREIGN KEY (contacttemplate_id) REFERENCES contact_templates(contacttemplate_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# contact groups
	#

	$dbh->do("CREATE TABLE contactgroups (contactgroup_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			alias varchar(255) NOT NULL,
			comment TEXT) TYPE=INNODB");

	#
	####################################################################################
	# contacts
	#

	$dbh->do("CREATE TABLE contacts (contact_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			alias varchar(255) NOT NULL,
			email VARCHAR(50),
			pager VARCHAR(50),
			contacttemplate_id SMALLINT(4) UNSIGNED,
			status TINYINT(1),
			comment TEXT) TYPE=INNODB");
	#
	####################################################################################
	# contact overrides
	#

	$dbh->do("CREATE TABLE contact_overrides (contact_id SMALLINT(4) UNSIGNED PRIMARY KEY,
			host_notification_period SMALLINT(4) UNSIGNED,
			service_notification_period SMALLINT(4) UNSIGNED,
			data TEXT,
			FOREIGN KEY (contact_id) REFERENCES contacts(contact_id) ON DELETE CASCADE)	TYPE=INNODB");

	#
	####################################################################################
	# contact commands overrides
	#

	$dbh->do("CREATE TABLE contact_command_overrides (contact_id SMALLINT(4) UNSIGNED,
			type VARCHAR(50),
			command_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (contact_id,type,command_id),
			FOREIGN KEY (command_id) REFERENCES commands(command_id) ON DELETE CASCADE,
			FOREIGN KEY (contact_id) REFERENCES contacts(contact_id) ON DELETE CASCADE) TYPE=INNODB");
	#
	####################################################################################
	# contact group Members
	#

	$dbh->do("CREATE TABLE contactgroup_contact (contactgroup_id SMALLINT(4) UNSIGNED,
			contact_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (contactgroup_id,contact_id),
			FOREIGN KEY (contact_id) REFERENCES contacts(contact_id) ON DELETE CASCADE,
			FOREIGN KEY (contactgroup_id) REFERENCES contactgroups(contactgroup_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# contact group Assign
	#

	$dbh->do("CREATE TABLE contactgroup_assign (contactgroup_id SMALLINT(4) UNSIGNED,
			type VARCHAR(50) NOT NULL,
			object SMALLINT(4) UNSIGNED,
			PRIMARY KEY (contactgroup_id,type,object)) TYPE=INNODB");

	#
	####################################################################################
	# Host Templates
	#

	$dbh->do("CREATE TABLE host_templates (hosttemplate_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			check_period SMALLINT(4) UNSIGNED,
			notification_period SMALLINT(4) UNSIGNED,
			check_command SMALLINT(4) UNSIGNED,
			event_handler SMALLINT(4) UNSIGNED,
			data TEXT,
			comment TEXT)");

	#
	####################################################################################
	# Hosts
	#

	$dbh->do("CREATE TABLE hosts (host_id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			alias varchar(255) NOT NULL,
			address VARCHAR(50) NOT NULL,
			os VARCHAR(50),
			hosttemplate_id SMALLINT(4) UNSIGNED,
			hostextinfo_id SMALLINT(4) UNSIGNED,
			hostprofile_id SMALLINT(4) UNSIGNED,
			serviceprofile_id SMALLINT(4) UNSIGNED,
			host_escalation_id SMALLINT(4) UNSIGNED,
			service_escalation_id SMALLINT(4) UNSIGNED,
			status TINYINT(1),
			comment TEXT) TYPE=INNODB");

	$dbh->do("CREATE TABLE host_overrides (host_id INT(6) UNSIGNED PRIMARY KEY,
			check_period SMALLINT(4) UNSIGNED,
			notification_period SMALLINT(4) UNSIGNED,
			check_command SMALLINT(4) UNSIGNED,
			event_handler SMALLINT(4) UNSIGNED,
			data TEXT, 
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Host Parent Members
	#

	$dbh->do("CREATE TABLE host_parent (host_id INT(6) UNSIGNED,
			parent_id INT(6) UNSIGNED,
			PRIMARY KEY (host_id,parent_id),
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE,
			FOREIGN KEY (parent_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Hostgroups
	#

	$dbh->do("CREATE TABLE hostgroups (hostgroup_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			alias varchar(255) NOT NULL,
			hostgroup_escalation_id SMALLINT(4) UNSIGNED,
			host_escalation_id SMALLINT(4) UNSIGNED,
			service_escalation_id SMALLINT(4) UNSIGNED,
			status TINYINT(1),
			comment TEXT) TYPE=INNODB");

	#
	####################################################################################
	# Hostgroup Host
	#

	$dbh->do("CREATE TABLE hostgroup_host (hostgroup_id SMALLINT(4) UNSIGNED,
			host_id INT(6) UNSIGNED,
			PRIMARY KEY (hostgroup_id,host_id),
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE,
			FOREIGN KEY (hostgroup_id) REFERENCES hostgroups(hostgroup_id) ON DELETE CASCADE) TYPE=INNODB");
			
	#
	####################################################################################
	# Service Templates
	#

	$dbh->do("CREATE TABLE service_templates (servicetemplate_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			parent_id SMALLINT(4) UNSIGNED,
			check_period SMALLINT(4) UNSIGNED,
			notification_period SMALLINT(4) UNSIGNED,
			check_command SMALLINT(4) UNSIGNED,
			event_handler SMALLINT(4) UNSIGNED,
			data TEXT,
			comment TEXT)");

	#
	####################################################################################
	# Services
	#

	$dbh->do("CREATE TABLE services (service_id INT(8) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			host_id INT(6) UNSIGNED NOT NULL,
			servicename_id SMALLINT(4) UNSIGNED NOT NULL,
			servicetemplate_id SMALLINT(4) UNSIGNED,
			serviceextinfo_id SMALLINT(4) UNSIGNED,
			escalation_id SMALLINT(4) UNSIGNED,
			status TINYINT(1),
			check_command SMALLINT(4) UNSIGNED,
			command_line TEXT,
			comment TEXT, 
			INDEX (host_id),
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Services overrides
	#

	$dbh->do("CREATE TABLE service_overrides (service_id INT(8) UNSIGNED PRIMARY KEY,
			check_period SMALLINT(4) UNSIGNED,
			notification_period SMALLINT(4) UNSIGNED,
			event_handler SMALLINT(4) UNSIGNED,
			data TEXT,
			FOREIGN KEY (service_id) REFERENCES services(service_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Servicegroups
	#

	$dbh->do("CREATE TABLE servicegroups (servicegroup_id INT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			alias varchar(255) NOT NULL) TYPE=INNODB");

	#
	####################################################################################
	# Servicegroups Service
	#

	$dbh->do("CREATE TABLE servicegroup_service (servicegroup_id INT(6) UNSIGNED,
			host_id INT(6) UNSIGNED,
			service_id INT(8) UNSIGNED,
			PRIMARY KEY (servicegroup_id,host_id,service_id),
			FOREIGN KEY (servicegroup_id) REFERENCES servicegroups(servicegroup_id) ON DELETE CASCADE,
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE,
			FOREIGN KEY (service_id) REFERENCES services(service_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Time Periods
	#

	$dbh->do("CREATE TABLE time_periods (timeperiod_id INT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			alias varchar(255) NOT NULL,
			data TEXT,
			comment TEXT)");

	#
	####################################################################################
	# Extended Host Info Template
	#

	$dbh->do("CREATE TABLE extended_host_info_templates (hostextinfo_id INT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			data TEXT,
			script VARCHAR(255),
			comment TEXT)");

	#
	####################################################################################
	# Extended Info Coords
	#

	$dbh->do("CREATE TABLE extended_info_coords (host_id INT(4) UNSIGNED PRIMARY KEY,
			data TEXT,
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Extended Service Info Templates
	#

	$dbh->do("CREATE TABLE extended_service_info_templates (serviceextinfo_id INT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			data TEXT,
			script VARCHAR(255),
			comment TEXT)");

	#
	####################################################################################
	# Service Dependencies
	#

	$dbh->do("CREATE TABLE service_dependency (id INT(8) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			service_id INT(8) UNSIGNED NOT NULL,
			host_id INT(6) UNSIGNED NOT NULL,
			depend_on_host_id INT(6) UNSIGNED NOT NULL,
			template SMALLINT(4) UNSIGNED NOT NULL,
			comment TEXT,
			INDEX (service_id),
			FOREIGN KEY (service_id) REFERENCES services(service_id) ON DELETE CASCADE,
			FOREIGN KEY (depend_on_host_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Service Dependencies Templates
	#

	$dbh->do("CREATE TABLE service_dependency_templates (id INT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			servicename_id SMALLINT(4) UNSIGNED NOT NULL,
			data TEXT,
			comment TEXT)");

	#
	####################################################################################
	# Host Dependencies
	#

	$dbh->do("CREATE TABLE host_dependencies (host_id INT(6) UNSIGNED,
			parent_id INT(6) UNSIGNED,
			data TEXT,
			comment TEXT,
			PRIMARY KEY (host_id,parent_id),
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE,
			FOREIGN KEY (parent_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");


	#
	####################################################################################
	# Setup
	#

	$dbh->do("CREATE TABLE setup (name VARCHAR(255) PRIMARY KEY,
			type VARCHAR(50),
			value TEXT)");

	#
	####################################################################################
	# Users
	#

	$dbh->do("CREATE TABLE users (user_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			user_acct VARCHAR(20) NOT NULL,
			user_name VARCHAR(255) NOT NULL,
			password VARCHAR(20) NOT NULL,
			session INT(10) UNSIGNED) TYPE=INNODB");

	#
	####################################################################################
	# User groups
	#

	$dbh->do("CREATE TABLE user_groups (usergroup_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) NOT NULL,
			description VARCHAR(100)) TYPE=INNODB");

	#
	####################################################################################
	# User - group
	#

	$dbh->do("CREATE TABLE user_group (usergroup_id SMALLINT(4) UNSIGNED,
			user_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (usergroup_id,user_id),
			FOREIGN KEY (user_id) REFERENCES users(user_id) ON DELETE CASCADE,
			FOREIGN KEY (usergroup_id) REFERENCES user_groups(usergroup_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Access List
	#

	$dbh->do("CREATE TABLE access_list (object VARCHAR(50),
			type VARCHAR(50),
			usergroup_id SMALLINT(4) UNSIGNED,
			access_values VARCHAR(20),
			PRIMARY KEY (object,type,usergroup_id),
			FOREIGN KEY (usergroup_id) REFERENCES user_groups(usergroup_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Import Schemas
	#

	$dbh->do("CREATE TABLE import_schemas (name VARCHAR(255) PRIMARY KEY,
			field_separator VARCHAR(50),
			host CHAR(1),
			alias CHAR(1),
			address CHAR(1),
			os CHAR(1),
			service CHAR(1),
			info CHAR(1))");

	#
	####################################################################################
	# Match Strings
	#


	$dbh->do("CREATE TABLE match_strings (id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			string VARCHAR(50),
			type VARCHAR(20),
			class VARCHAR(50),
			object VARCHAR(50))");

	#
	####################################################################################
	# Stage Host
	#

	$dbh->do("CREATE TABLE stage_hosts (name VARCHAR(255),
			user_acct VARCHAR(50),
			PRIMARY KEY (name,user_acct),
			type VARCHAR(20),
			status TINYINT(1),
			alias varchar(255),
			address VARCHAR(50),
			os VARCHAR(50),
			hostprofile VARCHAR(50),
			serviceprofile VARCHAR(50),
			info VARCHAR(50))");

	#
	####################################################################################
	# Stage Host-Services
	#


	$dbh->do("CREATE TABLE stage_host_services (name VARCHAR(255),
			user_acct VARCHAR(50),
			host VARCHAR(50),
			PRIMARY KEY (name,user_acct,host),
			type VARCHAR(20),
			status TINYINT(1),
			service_id INT UNSIGNED)");

	#
	####################################################################################
	# Stage Host-Hostgroups
	#


	$dbh->do("CREATE TABLE stage_host_hostgroups (name VARCHAR(255),
			user_acct VARCHAR(50),
			hostgroup VARCHAR(50),
			PRIMARY KEY (name,user_acct,hostgroup))");

	#
	####################################################################################
	# Stage Other
	#

	$dbh->do("CREATE TABLE stage_other (name VARCHAR(255),
			type VARCHAR(50),
			parent VARCHAR(255),
			PRIMARY KEY (name,type,parent),
			data TEXT,
			comment TEXT)");


	#
	####################################################################################
	# Stage Status
	#

	$dbh->do("CREATE TABLE stage_status (id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			type VARCHAR(50),
			message VARCHAR(255))");

	#
	####################################################################################
	# Stage Escalataions
	#

	$dbh->do("CREATE TABLE stage_escalations (id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			objects TEXT,
			template SMALLINT(4),
			contactgroups VARCHAR(50),
			first_notify SMALLINT(4),
			last_notify SMALLINT(4),
			interval_notify SMALLINT(4))");

	#
	####################################################################################
	# Service Names
	#


	$dbh->do("CREATE TABLE service_names (servicename_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(250) UNIQUE NOT NULL,
			description VARCHAR(100),
			template SMALLINT(4) UNSIGNED,
			check_command SMALLINT(4) UNSIGNED,
			command_line TEXT,
			dependency SMALLINT(4) UNSIGNED,
			escalation SMALLINT(4) UNSIGNED,
			extinfo SMALLINT(4) UNSIGNED) TYPE=INNODB");

	#
	####################################################################################
	# Service Profiles
	#

	$dbh->do("CREATE TABLE profiles_service (serviceprofile_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(250) UNIQUE NOT NULL,
			description VARCHAR(100),
			file_id SMALLINT(4) UNSIGNED) TYPE=INNODB");

	#
	####################################################################################
	# Service Name Service Profile
	#

	$dbh->do("CREATE TABLE serviceprofile (servicename_id SMALLINT(4) UNSIGNED,
			serviceprofile_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (servicename_id,serviceprofile_id)) TYPE=INNODB");

	#
	####################################################################################
	# Host Profiles
	#

	$dbh->do("CREATE TABLE profiles_host (hostprofile_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			description VARCHAR(255),
			host_template_id SMALLINT(4) UNSIGNED,
			host_extinfo_id SMALLINT(4) UNSIGNED,
			host_escalation_id SMALLINT(4) UNSIGNED,
			service_escalation_id SMALLINT(4) UNSIGNED,
			serviceprofile_id SMALLINT(4) UNSIGNED,
			file_id SMALLINT(4) UNSIGNED) TYPE=INNODB");

	#
	####################################################################################
	# Service Profile Host Profile
	#

	$dbh->do("CREATE TABLE profile_host_profile_service (hostprofile_id SMALLINT(4) UNSIGNED,
			serviceprofile_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (hostprofile_id,serviceprofile_id),
			FOREIGN KEY (serviceprofile_id) REFERENCES profiles_service(serviceprofile_id) ON DELETE CASCADE,
			FOREIGN KEY (hostprofile_id) REFERENCES profiles_host(hostprofile_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Service Profile Host 
	#

	$dbh->do("CREATE TABLE profile_service_host (serviceprofile_id SMALLINT(4) UNSIGNED,
			host_id INT(6) UNSIGNED,
			PRIMARY KEY (serviceprofile_id,host_id),
			FOREIGN KEY (serviceprofile_id) REFERENCES profiles_service(serviceprofile_id) ON DELETE CASCADE,
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Profile Hostgroup
	#

	$dbh->do("CREATE TABLE profile_hostgroup (hostprofile_id SMALLINT(4) UNSIGNED,
			hostgroup_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (hostprofile_id,hostgroup_id),
			FOREIGN KEY (hostgroup_id) REFERENCES hostgroups(hostgroup_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Profile Parent
	#

	$dbh->do("CREATE TABLE profile_parent (hostprofile_id SMALLINT(4) UNSIGNED,
			host_id INT(6) UNSIGNED,
			PRIMARY KEY (hostprofile_id,host_id),
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Files
	#

	$dbh->do("CREATE TABLE files (file_id SMALLINT(4) UNSIGNED AUTO_INCREMENT,
			name VARCHAR(255),
			path VARCHAR(255),
			type VARCHAR(50),
			PRIMARY KEY (file_id,name,path))");


	#
	####################################################################################
	# File Host
	#

	$dbh->do("CREATE TABLE file_host (file_id SMALLINT(4) UNSIGNED,
			host_id INT(6) UNSIGNED,
			PRIMARY KEY (file_id,host_id),
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# File Service
	#

	$dbh->do("CREATE TABLE file_service (file_id SMALLINT(4) UNSIGNED,
			service_id INT(4) UNSIGNED,
			PRIMARY KEY (file_id,service_id),
			FOREIGN KEY (service_id) REFERENCES services(service_id) ON DELETE CASCADE) TYPE=INNODB");


	#
	####################################################################################
	# Escalation Templates
	#

	$dbh->do("CREATE TABLE escalation_templates (template_id SMALLINT(4) UNSIGNED AUTO_INCREMENT,
			name VARCHAR(255) UNIQUE NOT NULL,
			type VARCHAR(50) NOT NULL,
			data TEXT,
			comment TEXT,
			servicename_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (template_id,name,type),
			FOREIGN KEY (servicename_id) REFERENCES service_names(servicename_id) ON DELETE CASCADE) TYPE=INNODB");

	#
	####################################################################################
	# Escalation Trees
	#


	$dbh->do("CREATE TABLE escalation_trees (tree_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255) UNIQUE NOT NULL,
			description VARCHAR(100),
			type VARCHAR(50) NOT NULL)");

	#
	####################################################################################
	# Escalation Trees Template
	#

	$dbh->do("CREATE TABLE escalation_tree_template (tree_id SMALLINT(4) UNSIGNED,
			template_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (tree_id,template_id))");

	#
	####################################################################################
	# Escalation Tree Template Contactgroup
	#

	$dbh->do("CREATE TABLE tree_template_contactgroup (tree_id SMALLINT(4) UNSIGNED,
			template_id SMALLINT(4) UNSIGNED,
			contactgroup_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (tree_id,template_id,contactgroup_id))");

	#
	####################################################################################
	# Externals
	#

	$dbh->do("CREATE TABLE externals (external_id SMALLINT(4) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
			name VARCHAR(255),
			description VARCHAR(50),
			type VARCHAR(20) NOT NULL,
			display TEXT,
			handler TEXT) TYPE=INNODB");

	$dbh->do("CREATE TABLE external_host (external_id SMALLINT(4) UNSIGNED,
			host_id INT(6) UNSIGNED NOT NULL,
			data TEXT,
			PRIMARY KEY (external_id,host_id),
			FOREIGN KEY (external_id) REFERENCES externals(external_id) ON DELETE CASCADE,
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE) TYPE=INNODB");

	$dbh->do("CREATE TABLE external_service (external_id SMALLINT(4) UNSIGNED,
			host_id INT(6) UNSIGNED NOT NULL,
			service_id INT(8) UNSIGNED NOT NULL,
			data TEXT,
			PRIMARY KEY (external_id,host_id,service_id),
			FOREIGN KEY (external_id) REFERENCES externals(external_id) ON DELETE CASCADE,
			FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE CASCADE,
			FOREIGN KEY (service_id) REFERENCES services(service_id) ON DELETE CASCADE) TYPE=INNODB");

	$dbh->do("CREATE TABLE external_host_profile (external_id SMALLINT(4) UNSIGNED,
			hostprofile_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (external_id,hostprofile_id),
			FOREIGN KEY (external_id) REFERENCES externals(external_id) ON DELETE CASCADE,
			FOREIGN KEY (hostprofile_id) REFERENCES profiles_host(hostprofile_id) ON DELETE CASCADE) TYPE=INNODB");

	$dbh->do("CREATE TABLE external_service_names (external_id SMALLINT(4) UNSIGNED,
			servicename_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (external_id,servicename_id),
			FOREIGN KEY (external_id) REFERENCES externals(external_id) ON DELETE CASCADE,
			FOREIGN KEY (servicename_id) REFERENCES service_names(servicename_id) ON DELETE CASCADE) TYPE=INNODB");


	#
	####################################################################################
	# Monarch 
	####################################################################################
	# Load default data
	#

	my @asset_list = ('host_templates','host_dependencies','extended_host_info_templates','hostgroups',
	'service_templates','service_dependency_templates','extended_service_info_templates','contact_templates','contacts','contactgroups','time_periods','commands',
	'profiles','hosts','escalations','services','parent_child','export','servicegroups');

	my @options = ('login_authentication','session_timeout','super_user_password');
	foreach my $opt (@options) {
		my $timeout = undef;
		if ($opt eq 'session_timeout') { $timeout = 3600 }
		my $sqlstmt = "insert into setup values('$opt','','$timeout')";
		my $sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;
	}

	my $sqlstmt = "insert into user_groups values('','super_users','System defined group granted complete access.')";
	my $sth = $dbh->prepare($sqlstmt);
	unless ($sth->execute()) {
		print "\n\n $sqlstmt $@\n\n";
		my $x = <STDIN>;
	}

	$sqlstmt = "select max(usergroup_id) as lastrow from user_groups";
	my $gid = $dbh->selectrow_array($sqlstmt);

	my $now = time;
	my @saltchars = ('a'..'z','A'..'Z','0'..'9',',','/');
	srand(time() ^ ($$ + ($$ << 15)));
	my $salt = $saltchars[int(rand(64))];
	$salt .= $saltchars[int(rand(64))];
	my $newpw = crypt('password', $salt);

	$sqlstmt = "insert into users values ('','super_user','Super User Account','$newpw','$now')";
	$sth = $dbh->prepare($sqlstmt);
	unless ($sth->execute()) {
		print "\n\n $sqlstmt $@\n\n";
		my $x = <STDIN>;
	}
	$sth->finish;

	$sqlstmt = "select max(user_id) as lastrow from users";
	my $uid = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "insert into user_group values ('$gid','$uid')";
	$sth = $dbh->prepare($sqlstmt);
	unless ($sth->execute()) {
		print "\n\n $sqlstmt $@\n\n";
		my $x = <STDIN>;
	}
	$sth->finish;

	foreach my $asset (@asset_list) {
		my $sqlstmt = "insert into access_list values('$asset','design_manage','$gid','add,modify,delete')";
		my $sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;
	}

	my @control_list = ('users','user_groups','setup','nagios_cgi_configuration','nagios_main_configuration','files','nagios_resource_macros','load','pre_flight_test','commit','run_external_scripts');
	foreach my $asset (@control_list) {
		my $sqlstmt = "insert into access_list values('$asset','control','$gid','full_control')";
		my $sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;
	}

	my @discover_list = ('nmap','import','match_strings','process_stage');
	foreach my $asset (@discover_list) {
		my $sqlstmt = "insert into access_list values('$asset','discover','$gid','full_control')";
		my $sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;
	}

	for (my $i = 1;$i <= 32;$i++) {
		my $sqlstmt = "insert into setup values('resource_label$i','resource','')";
		my $sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;		
		$sqlstmt = "insert into setup values('user$i','resource','')";
		$sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;



	}

	my %file_object = ('commands' => 'checkcommands.cfg',
		'miscommands' => 'miscommands.cfg',
		'contact_templates' => 'contact_templates.cfg',
		'contactgroups' => 'contactgroups.cfg',
		'contacts' => 'contacts.cfg',
		'escalation_templates' => 'escalation_templates.cfg',
		'escalations' => 'escalations.cfg',
		'extended_host_info_templates' => 'extended_host_info_templates.cfg',
		'extended_host_info' => 'extended_host_info.cfg',
		'extended_service_info_templates' => 'extended_service_info_templates.cfg',
		'extended_service_info' => 'extended_service_info.cfg',
		'host_dependencies' => 'host_dependencies.cfg',
		'host_templates' => 'host_templates.cfg',
		'hostgroups' => 'hostgroups.cfg',
		'service_dependency' => 'service_dependency.cfg',
		'service_dependency_templates' => 'service_dependency_templates.cfg',
		'service_templates' => 'service_templates.cfg',
		'servicegroups' => 'servicegroups.cfg',
		'time_periods' => 'time_periods.cfg',
		'resource' => 'resource.cfg',
		'services' => 'services.cfg',
		'hosts' => 'hosts.cfg');

	foreach my $obj (keys %file_object) {
		my $sqlstmt = "insert into files values('','$file_object{$obj}','$nagios_etc','$obj')";
		my $sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;
		$sqlstmt = "select max(file_id) as lastrow from files";
		my $file_id = $dbh->selectrow_array($sqlstmt);
		my $sqlstmt = "insert into setup values('$obj','file','$file_id')";
		my $sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;
	}


	my %misc = ('nagios_version' => $nagios_ver,'nagios_etc' => $nagios_etc,'backup_home' => $monarch_home.'/backup',
		'monarch_version' => $version,'monarch_home' => $monarch_home,'upload_home' => '/tmp','nagios_bin' => $nagios_bin,
		'doc_root' => $doc_root,'cgi_bin' => $cgi_bin);
	foreach my $key (keys %misc) {
		my $sqlstmt = "insert into setup values('$key','config','$misc{$key}')";
		my $sth = $dbh->prepare($sqlstmt);
		unless ($sth->execute()) {
			print "\n\n $sqlstmt $@\n\n";
			my $x = <STDIN>;
		}
		$sth->finish;
	}
	if ($dbh) {
	   $dbh and $dbh->disconnect();
	}
}

sub distribution() {
	umask 0002;
	if (! -e "$doc_root/monarch") {
		mkdir("$doc_root/monarch", 0770) || print "\n\n$doc_root/monarch $!";
	}
	if (! -e "$doc_root/monarch/download") {
		mkdir("$doc_root/monarch/download", 0770) || print "\n\n$doc_root/monarch/download $!";
	}
	if (! -e "$doc_root/monarch/images") {
		mkdir("$doc_root/monarch/images", 0770) || print "\n\n$doc_root/monarch/images $!";
	}
	if (! -e "$doc_root/monarch/doc") {
		mkdir("$doc_root/monarch/doc", 0770) || print "\n\n$doc_root/monarch/doc $!";
	}
	my $path = '/';
	my @folders = split(/\//, $monarch_home);
	foreach my $folder (@folders) {
		if (! -e "$path$folder") {
			mkdir("$path$folder", 0755) || print "\n\n$path$folder $!";
		}
		$path .= "$folder/";
	}
	if (! -e "$monarch_home/workspace") {
		mkdir("$monarch_home/workspace", 0770) || print "\n\n$monarch_home/monarch/workspace $!";
	}
	if (! -e "$monarch_home/upload") {
		mkdir("$monarch_home/upload", 0770) || print "\n\n$monarch_home/upload $!";
	}
	if (! -e "$monarch_home/backup") {
		mkdir("$monarch_home/backup", 0770) || print "\n\n$monarch_home/backup $!";
	}
	if (! -e "$monarch_home/bin") {
		mkdir("$monarch_home/bin", 0770) || print "\n\n$monarch_home/bin $!";
	}	
	if (! -e "$monarch_home/lib") {
		mkdir("$monarch_home/lib", 0770) || print "\n\n$monarch_home/lib $!";
	}	
	if (! -e "$monarch_home/conf") {
		mkdir("$monarch_home/conf", 0770) || print "\n\n$monarch_home/conf $!";
	}	
	if (! -e "$monarch_home/tools") {
		mkdir("$monarch_home/tools", 0770) || print "\n\n$monarch_home/tools $!";
	}	
	opendir(DIR, './images') || print "\n\n./images $!";
	while (my $file = readdir(DIR)) {
		if ($file =~/^\./) { next }
		copy("./images/$file","$doc_root/monarch/images/$file") || print "\n\n./images/$file $doc_root/monarch/images/$file $!";
	}
	close(DIR);
	opendir(DIR, './doc') || print "\n\n./doc $!";
	while (my $file = readdir(DIR)) {
		if ($file =~/^\./) { next }
		copy("./doc/$file","$doc_root/monarch/doc/$file") || print "\n\n./doc/$file $doc_root/monarch/doc/$file $!";
	}
	close(DIR);
	open(FILE, "< ./monarch.cgi") || print "\n\n./monarch.cgi $!";
	my $out_to_file = undef;
	while (my $line = <FILE>) {
		if ($line =~ /STRINGTOREPLACE/) {
			$line =~ s/STRINGTOREPLACE/$monarch_home/;
		}
		$out_to_file .= $line;
	}
	close(FILE);
	open(FILE, "> $cgi_bin/monarch.cgi") || print "\n$cgi_bin/monarch.cgi $!";
	print FILE $out_to_file;
	close(FILE);

	my @files = ('monarch.js','nicetitle.js','monarch.css','ua.js','ftiens4.js');
	foreach my $file (@files) {
		copy("./$file","$doc_root/monarch/$file") || print "\n\n./$file $doc_root/monarch/$file $!";
	}

	open(FILE, "< ./MonarchConf.pm") || print "\n\n./MonarchConf.pm $!";
	my $out_to_file = undef;
	while (my $line = <FILE>) {
		if ($line =~ /DATABASE/) {
			$line =~ s/DATABASE/$database/;
		} elsif ($line =~ /DBHOST/) {
			$line =~ s/DBHOST/$databasehost/;
		} elsif ($line =~ /DBUSER/) {
			$line =~ s/DBUSER/$databaseuser/;
		} elsif ($line =~ /DBPASS/) {
			$line =~ s/DBPASS/$databasepasswd/;
		}
		$out_to_file .= $line;
	}

	close(FILE);
	open(FILE, "> $monarch_home/lib/MonarchConf.pm") || print "\n$monarch_home/lib/MonarchConf.pm $!";
	print FILE $out_to_file;
	close(FILE);

	@files = ('MonarchFile.pm','MonarchForms.pm','MonarchStorProc.pm','MonarchExternals.pm','MonarchDoc.pm','MonarchLoad.pm');
	foreach my $file (@files) {
		copy("./$file","$monarch_home/lib/$file") || print "\n\nnot copied ./$file $monarch_home/lib/$file $!";
	}
	@files = ('nagios_reload');
	foreach my $file (@files) {
		copy("./$file","$monarch_home/bin/$file") || print "\n\nnot copied ./$file $monarch_home/bin/$file $!\n";
	}
	system("chown $web_user:$web_group $cgi_bin/monarch.cgi");
	system("chmod 755 $cgi_bin/monarch.cgi");
	system("chown -R $web_user:$web_group $doc_root/monarch");
	system("chown -R $web_user:$web_group $monarch_home");
	system("chown root:$web_group $monarch_home/bin/nagios_reload");
	system("chmod 4750 $monarch_home/bin/nagios_reload");
	my $errors = 0;
	if ($set_perms =~ /yes/i) {
		my @files = ();
		my @dirs = ();
		my $resource_file = undef;
		if (-e "$nagios_etc/nagios.cfg") {
			system("chmod g+rw $nagios_etc/nagios.cfg");
			open (FILE, "< $nagios_etc/nagios.cfg") || print "\n$nagios_etc/nagios.cfg $!";
			while (my $line = <FILE>) {
				if ($line =~ /^cfg_file=(\S+)$/) { push @files, $1 }
				if ($line =~ /^cfg_dir=(\S+)$/) { push @dirs, $1 }
				if ($line =~ /^resource_file=(\S+)$/) { $resource_file = $1 }
			}					
			close(FILE);
			foreach my $dir (@dirs) {
				opendir(DIR, $dir) || print "\nerror: cannot open $dir to read $!";
				while (my $file = readdir(DIR)) {
					if ($file =~ /(\S+\.cfg$)/) {  push @files, "$dir/$file" }
				}
				close(DIR);
				system("chmod g+rwx $dir");
			}
		} else {
			print "\n\n\tError: $nagios_etc/nagios.cfg does not exist\n";
			$errors = 1;
		}
		if (-e "$nagios_etc/cgi.cfg") {
			system("chmod g+rw $nagios_etc/cgi.cfg");
			open(FILE, "< $nagios_etc/cgi.cfg") or print "error: cannot open $nagios_etc/cgi.cfg to read $!";
			while (my $line = <FILE>) {
				if ($line =~ /^xedtemplate_config_file=(\S+)$/) { push @files, $1 }
			}
			close(FILE);

		} else {
			print "\n\n\tError: $nagios_etc/cgi.cfg does not exist\n";
			$errors = 1;
		}
		unless ($errors) {
			foreach my $file (@files) {
				system("chmod g+rw $file");
			}
		}	
		if ($resource_file =~ /(.*)\/.*\.cfg/) {
			system("chmod g+rwx $1");
		}
		system("chmod g+rw $resource_file");
		system("chmod g+rwx $nagios_etc");
	}	
}

sub collect_info() {
	my $validated = 0;
	print "\n\n\tThis script will help you setup Monarch on your system.";
	print "\n\n\tDefaults and guesses will be provided where possible.\n";
	print "\n\n\n\tWhat is the host name of your MySQL database server? [$databasehost] : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $databasehost = $input }
	print "\n\n\n\tWhat is the name of the user that will have access to the database? [$databaseuser] : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $databaseuser = $input }
	print "\n\n\n\tWhat is the password for user $databaseuser to access the database [$databasepasswd] : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $databasepasswd = $input }
	print "\n\n\n\tWhat is the name of the database you will use for your Monarch installation? [$database] : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $database = $input }
	$validated = 0;
	until ($validated) {
		print "\n\n\n\tEnter web server's user account [$web_user] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $web_user = $input }
		my @user = getpwnam($web_user);
		if ($user[0]) { 
			$validated = 1;
		} else {
			print "\n\n\n\tError: Invalid, user $input does not exist.";
		}
	}
	$validated = 0;
	until ($validated) {
		print "\n\n\n\tEnter web server's user group [$web_group] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $web_group = $input }
		my @grp = getgrnam($web_group);
		if ($grp[0]) { 
			$validated = 1;
		} else {
			print "\n\n\n\tError: Invalid, group $input does not exist.";
		}
	}

	unless (-e $doc_root) { 
		if (-e "/var/www/htdocs") {
			$doc_root = "/var/www/htdocs";
		} else {
			$doc_root = "unknown";
		}
	} 
	$validated = 0;
	until ($validated) {
		if ($doc_root) {
			print "\n\n\n\tWhat is your web server's document root [$doc_root] : ";
		} else {
			print "\n\n\n\tWhat is the full path to your web server's document root : ";
		}
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $doc_root = $input }
		if (-e $doc_root) { 
			$validated = 1;
		} else {
			print "\n\n\n\tError: Invalid entry $doc_root does not exist.";
		}
	}

	unless (-e $cgi_bin) { $cgi_bin = "unknown" }
	$validated = 0;
	until ($validated) {
		print "\n\n\n\tEnter the full path of your cgi-bin directory [$cgi_bin] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $cgi_bin = $input }
		if (-e $cgi_bin) { 
			$validated = 1;
		} else {
			print "\n\n\n\tError: Invalid entry $doc_root does not exist.";
		}
	}

	print "\n\n\n\tEnter the full installation path for Monarch [$monarch_home] : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $monarch_home = $input }

	if (-e '/etc/nagios/nagios.cfg') {
		$nagios_etc = '/etc/nagios';
	} elsif (-e '/usr/local/nagios/etc/nagios.cfg') {
		$nagios_etc = '/usr/local/nagios/etc';
	} else {
		$nagios_etc = undef;
	}

	$validated = 0;
	until ($validated) {
		print "\n\n\n\tEnter the full path to your nagios.cfg file [$nagios_etc] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $nagios_etc = $input }
		if (-e $nagios_etc) { 
			$validated = 1;
		} else {
			print "\n\n\n\tError: Invalid entry $nagios_etc does not exist.";
		}
	}

	if (-e '/usr/sbin/nagios') {
		$nagios_bin = '/usr/sbin';
	} elsif (-e '/usr/local/nagios/bin/nagios') {
		$nagios_bin = '/usr/local/nagios/bin';
	} else {
		$nagios_bin = undef;
	}

	$validated = 0;
	until ($validated) {
		print "\n\n\n\tEnter the full path to your nagios binary file [$nagios_bin] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $nagios_bin = $input }
		if (-e $nagios_bin) {
			$validated = 1;
		} else {
			print "\n\n\n\tError: Invalid entry $nagios_bin does not exist.";
		}
	}
	if (-e "$nagios_etc/nagios.cfg") {
		my @stats = stat("$nagios_etc/nagios.cfg");
		if ($stats[5] eq '0') {
			print "\n\n\n\tWARNING: Group ownership of $nagios_etc/nagios.cfg is root.";
			print "\n\tYour web server account will need read and write access to the nagios object files."; 
			print "\n\tThe best method is to included the account in a non root group (nagios),";
			print "\n\tand make that group the owner of the nagios object files.";
		} else { 
			print "\n\n\n\tYour web server account will need read and write access to the nagios object files. ";
			print "\n\tWould you like to set group permissions so that Monarch can read your nagios cfg files including resource.cfg?\n\t(Note: You will need to manage this manually if your answer is no). yes|no : ";
			my $input = <STDIN>;
			chomp $input;
			if ($input =~ /^yes$/i) { 
				$set_perms = $input; 
			} else {
				$set_perms = 0; 
			}
		}
	}
	print qq(

		Database host	= $databasehost
		Database name	= $database
		Database user	= $databaseuser
		Database passwd = $databasepasswd
		Document root	= $doc_root
		Web user	= $web_user
		Web user group	= $web_group
		Web cgi-bin	= $cgi_bin
		Nagios bin	= $nagios_bin
		Nagios cfg dir	= $nagios_etc
		Install path	= $monarch_home

	Confirmation: Be absolutely certain these values are correct. 
	Does the information look correct? 'yes' or 'no' : );
	my $response = <STDIN>;
	if ($response =~ /yes/i) {
		if ($drop_tables == 1) { drop_tables() }
		distribution();
		create_db();
	} else {
		collect_info();
	}

}

check_prereqs();
collect_info();

print "\n\tSetup completed.\n\n";
