#!/usr/bin/perl --
# MonArch - Groundwork Monitor Architect
# monarch_update.pl
#
###############################################################################
# Release 0.97a
# 30-Sept-2005
###############################################################################
#

use DBI;
use strict;
use File::Copy;

my $version = '0.97a';
my $monarch_home = undef;
my $is_portal = 0;
my $monarch_home = 0;
my $nagios_bin = 0;
my $nagios_etc = 0;
my $cgi_bin = 0;
my $doc_root = 0;

print "\n\tMonarch $version Update";
print "\n=============================================================\n";
print "\n\tReading configuration file...\n";

until ($monarch_home) {
	if (-e "/usr/local/groundwork/config/db.properties") {
		$is_portal = 1;
		$monarch_home = "/usr/local/groundwork/monarch";
		$nagios_etc = "/usr/local/groundwork/nagios/etc";
		$nagios_bin = "/usr/local/groundwork/nagios/bin";
	} elsif (-e "/usr/local/groundwork/monarch/lib/MonarchConf.pm") {
		$monarch_home = "/usr/local/groundwork/monarch";
		print "\n\tPlease enter the Monarch installation path [ $monarch_home ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $monarch_home = $input }
		my $monarch_test = $monarch_home.'/lib/MonarchConf.pm';
		unless (-e $monarch_test) {
			print "\n\tError: Cannot locate MonarchConf.pm in path $monarch_home [/lib]...\n";
			$monarch_home = 0;
		}
	} else {
		print "\n\tPlease enter the Monarch installation path : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $monarch_home = $input }
		my $monarch_test = $monarch_home.'/lib/MonarchConf.pm';
		unless (-e $monarch_test) {
			print "\n\tError: Cannot locate MonarchConf.pm in path $monarch_home [/lib]...\n";
			$monarch_home = 0;
		}
	}
}

my ($dbhost, $database, $user, $passwd) = undef;
if ($is_portal) {
	open(FILE, "< /usr/local/groundwork/config/db.properties");
	while (my $line = <FILE>) {
		if ($line =~ /\s*monarch\.dbhost\s*=\s*(\S+)/) { $dbhost = $1 }
		if ($line =~ /\s*monarch\.database\s*=\s*(\S+)/) { $database = $1 }
		if ($line =~ /\s*monarch\.username\s*=\s*(\S+)/) { $user = $1 }
		if ($line =~ /\s*monarch\.password\s*=\s*(\S+)/) { $passwd = $1 }
	}
	close(FILE);
} else {
	open(FILE, "< $monarch_home/lib/MonarchConf.pm");
	while (my $line = <FILE>) {
		$line =~ s/\'|\"|;//g;
		if ($line =~ /\s*\$dbhost\s*=\s*(\S+)/) { $dbhost = $1 }
		if ($line =~ /\s*\$database\s*=\s*(\S+)/) { $database = $1 }
		if ($line =~ /\s*\$dbuser\s*=\s*(\S+)/) { $user = $1 }
		if ($line =~ /\s*\$dbpass\s*=\s*(\S+)/) { $passwd = $1 }
	}
	close(FILE);
}

print "\n\tConnecting to $database with user $user...\n";

my $dsn = "DBI:mysql:$database:$dbhost";
my $dbh = undef;
eval {$dbh = DBI->connect($dsn, $user, $passwd, {'RaiseError' => 1}) };
if ($@) {
	print "\nError: $@\n";
	die;
}

my $sqlstmt = "select value from setup where name = 'nagios_etc'";
my ($nagios_etc) = $dbh->selectrow_array($sqlstmt);

until ($nagios_etc) {
	if (-e "/etc/nagios/nagios.cfg") {
		$nagios_etc = "/etc/nagios";
		print "\n\tPlease enter the path to nagios.cfg [ $nagios_etc ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $nagios_etc = $input }
		unless (-e $nagios_etc) {
			print "\n\tError: Cannot locate nagios.cfg in path $nagios_etc...\n";
			$nagios_etc = 0;
		}
	} elsif (-e "/usr/local/nagios/etc/nagios.cfg") {
		$nagios_etc = "/usr/local/nagios/etc";
		print "\n\tPlease enter the path to nagios.cfg [ $nagios_etc ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $nagios_etc = $input }
		unless (-e $nagios_etc) {
			print "\n\tError: Cannot locate nagios.cfg in path $nagios_etc...\n";
			$nagios_etc = 0;
		}
	} else {
		print "\n\tPlease enter the path to nagios.cfg : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $nagios_etc = $input }
		unless (-e $nagios_etc) {
			print "\n\tError: Cannot locate nagios.cfg in path $nagios_etc...\n";
			$nagios_etc = 0;
		}
	}
}

$sqlstmt = "select value from setup where name = 'nagios_bin'";
my ($nagios_bin) = $dbh->selectrow_array($sqlstmt);

until ($nagios_bin) {
	if (-e "/usr/sbin/nagios") {
		$nagios_bin = "/usr/sbin";
		print "\n\tPlease enter the path to the nagios binary [ $nagios_bin ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $nagios_bin = $input }
		my $nagios_test = $nagios_bin.'/nagios';
		unless (-e $nagios_test) {
			print "\n\tError: Cannot locate nagios binary in path $nagios_bin...\n";
			$nagios_bin = 0;
		}
	} elsif (-e "/usr/local/nagios/bin") {
		$nagios_bin = "/usr/local/nagios/bin";
		print "\n\tPlease enter the path to the nagios binary [ $nagios_bin ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $nagios_bin = $input }
		my $nagios_test = $nagios_bin.'/nagios';
		unless (-e $nagios_test) {
			print "\n\tError: Cannot locate nagios binary in path $nagios_bin...\n";
			$nagios_bin = 0;
		}
	} else {
		print "\n\tPlease enter the path to nagios binary : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $nagios_bin = $input }
		my $nagios_test = $nagios_bin.'/nagios';
		unless (-e $nagios_test) {
			print "\n\tError: Cannot locate nagios binary in path $nagios_bin...";
			print "\n\tNote: If Nagios doesn't run on this machine try any valid path (i.e. /tmp).";
			$nagios_bin = 0;
		}
	}
}

$sqlstmt = "select value from setup where name = 'cgi_bin'";
my ($cgi_bin) = $dbh->selectrow_array($sqlstmt);

until ($cgi_bin) {
	if (-e "/var/www/cgi-bin") {
		$cgi_bin = "/var/www/cgi-bin";
		print "\n\tPlease enter the absolute cgi-bin location ex: '/var/www/cgi-bin' [ $cgi_bin ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $cgi_bin = $input }
		unless (-e $cgi_bin) {
			print "\n\tError: folder does not exist $cgi_bin...\n";
			$cgi_bin = 0;
		}
	} elsif (-e "/srv/www/htdocs") {
		$cgi_bin = "/srv/www/cgi-bin";
		print "\n\tPlease enter the absolute cgi-bin location ex: '/srv/www/cgi-bin' [ $cgi_bin ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $cgi_bin = $input }
		unless (-e $cgi_bin) {
			print "\n\tError: folder does not exist $cgi_bin...\n";
			$cgi_bin = 0;
		}
	} else {
		print "\n\tPlease enter the absolute cgi-bin location ex: '/var/www/cgi-bin' : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $cgi_bin = $input }
		unless (-e $cgi_bin) {
			print "\n\tError: folder does not exist $cgi_bin...\n";
			$cgi_bin = 0;
		}
	}
}

$sqlstmt = "select value from setup where name = 'doc_root'";
my ($doc_root) = $dbh->selectrow_array($sqlstmt);

until ($doc_root) {
	if (-e "/var/www/cgi-bin") {
		$doc_root = "/var/www/html";
		print "\n\tPlease enter the path to document root ex: '/var/www/html' [ $doc_root ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $doc_root = $input }
		unless (-e $doc_root) {
			print "\n\tError: folder does not exist $doc_root...\n";
			$doc_root = 0;
		}
	} elsif (-e "/srv/www/htdocs") {
		$doc_root = "/srv/www/htdocs";
		print "\n\tPlease enter the path to document root ex: '/var/www/html' [ $doc_root ] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $doc_root = $input }
		unless (-e $doc_root) {
			print "\n\tError: folder does not exist $doc_root...\n";
			$doc_root = 0;
		}
	} else {
		print "\n\tPlease enter the path to document root ex: '/var/www/htdocs' : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $doc_root = $input }
		unless (-e $doc_root) {
			print "\n\tError: folder does not exist $doc_root...\n";
			$doc_root = 0;
		}
	}
}	

print "\n\tUpdating setup information...\n";
my @names = ('monarch_version','resource_home','workspace_home','cgi_home','nagios_home','monarch_home',
	'nagios_etc','nagios_bin','doc_root','cgi_bin');
foreach my $name (@names) {
	my $sqlstmt = "delete from setup where name = '$name'"; 
	my $sth = $dbh->prepare ($sqlstmt);
	unless ($sth->execute) { print "\n\n\tError: $sqlstmt $@" }
}


	
$sqlstmt = "insert into setup values('monarch_version','config','$version')"; 
my $sth = $dbh->prepare ($sqlstmt);
unless ($sth->execute) { print "Error: $sqlstmt $@" }
$sqlstmt = "insert into setup values('monarch_home','config','$monarch_home')"; 
$sth = $dbh->prepare ($sqlstmt);
unless ($sth->execute) { print "Error: $sqlstmt $@" }
$sqlstmt = "insert into setup values('nagios_etc','config','$nagios_etc')"; 
$sth = $dbh->prepare ($sqlstmt);
unless ($sth->execute) { print "Error: $sqlstmt $@" }
$sqlstmt = "insert into setup values('nagios_bin','config','$nagios_bin')"; 
$sth = $dbh->prepare ($sqlstmt);
unless ($sth->execute) { print "Error: $sqlstmt $@" }
$sqlstmt = "insert into setup values('doc_root','config','$doc_root')"; 
$sth = $dbh->prepare ($sqlstmt);
unless ($sth->execute) { print "Error: $sqlstmt $@" }
$sqlstmt = "insert into setup values('cgi_bin','config','$cgi_bin')"; 
$sth = $dbh->prepare ($sqlstmt);
unless ($sth->execute) { print "Error: $sqlstmt $@" }

print "\n\tChecking tables...\n";
	
$sth = $dbh->prepare('show tables');
$sth->execute;
my %tables = ();
while(my @values = $sth->fetchrow_array()) {
	$tables{$values[0]} = 1;
}
$sth->finish;

#
# New Tables
#
####################################################################################
# contact overrides
#
unless ($tables{'contact_overrides'}) {
	$dbh->do("CREATE TABLE contact_overrides (contact_id SMALLINT(4) UNSIGNED PRIMARY KEY,
			host_notification_period SMALLINT(4) UNSIGNED,
			service_notification_period SMALLINT(4) UNSIGNED,
			data TEXT,
			FOREIGN KEY (contact_id) REFERENCES contacts(contact_id) ON DELETE CASCADE)	TYPE=INNODB");
}
#
####################################################################################
# contact commands overrides
#
unless ($tables{'contact_command_overrides'}) {
	$dbh->do("CREATE TABLE contact_command_overrides (contact_id SMALLINT(4) UNSIGNED,
			type VARCHAR(50),
			command_id SMALLINT(4) UNSIGNED,
			PRIMARY KEY (contact_id,type,command_id),
			FOREIGN KEY (command_id) REFERENCES commands(command_id) ON DELETE CASCADE,
			FOREIGN KEY (contact_id) REFERENCES contacts(contact_id) ON DELETE CASCADE) TYPE=INNODB");
}

#
# Modify Existing Tables
#
####################################################################################
# Change column types to text
#
my %table_text = ('services' => 'command_line','service_names' => 'command_line','setup' => 'value');
foreach my $table (keys %table_text) {
	$sqlstmt = "describe $table";
	my $sth = $dbh->prepare($sqlstmt);
	$sth->execute();
	my %fields = ();
	while(my @values = $sth->fetchrow_array()) {
		$fields{$values[0]} = $values[1];
	}
	$sth->finish;
	unless ($fields{$table_text{$table}} =~ /text/i) {
		$dbh->do("ALTER TABLE $table MODIFY $table_text{$table} TEXT");
	}
}
#
####################################################################################
# Change names to varchar 255
#

foreach my $table (keys %tables) {
	$sqlstmt = "describe $table";
	my $sth = $dbh->prepare($sqlstmt);
	$sth->execute();
	my %fields = ();
	while(my @values = $sth->fetchrow_array()) {
		$fields{$values[0]} = $values[1];
	}
	$sth->finish;
	if ($fields{'name'}) {
		unless ($fields{'name'} =~ /varchar\(255\)/i) {
			$dbh->do("ALTER TABLE $table MODIFY name varchar(255)");
		}
	}
	if ($fields{'alias'}) {
		unless ($fields{'name'} =~ /varchar\(255\)/i) {
			$dbh->do("ALTER TABLE $table MODIFY name varchar(255)");
		}
	}
}
# Add check_period for 2.0 
$sqlstmt = 'describe host_templates';
$sth = $dbh->prepare($sqlstmt);
$sth->execute();
my %fields = ();
while(my @values = $sth->fetchrow_array()) {
	$fields{$values[0]} = 1;
}
$sth->finish;	
unless (defined $fields{'check_period'}) {
	$dbh->do('alter table host_templates add check_period SMALLINT(4) UNSIGNED after name');
}

# change escalation template to add servicename_id column
$sqlstmt = 'describe escalation_templates';
$sth = $dbh->prepare($sqlstmt);
$sth->execute();
my %fields = ();
while(my @values = $sth->fetchrow_array()) {
	$fields{$values[0]} = 1;
}
$sth->finish;
unless (defined $fields{'servicename_id'}) {
	$dbh->do("ALTER TABLE escalation_templates add column (servicename_id SMALLINT(4) UNSIGNED)");
	$dbh->do("ALTER TABLE escalation_templates add FOREIGN KEY (servicename_id) REFERENCES service_names(servicename_id) ON DELETE CASCADE");
	$dbh->do("insert into service_names values('','*','special use','','','','','','')");
}

my $sqlstmt = "select value from setup where name = 'monarch_version'";
my ($monarch_ver) = $dbh->selectrow_array($sqlstmt);
unless ($monarch_ver) {
	print "\n\tConverting passwords...\n";
	$sqlstmt = 'select user_id, password from users';
	$sth = $dbh->prepare($sqlstmt);
	$sth->execute();
	my %fields = ();
	while(my @values = $sth->fetchrow_array()) {
		$fields{$values[0]} = 1;
	}
	$sth->finish;

	# encrypt passwords
	$sqlstmt = "select user_acct, password from users";
	$sth = $dbh->prepare($sqlstmt);
	$sth->execute();
	%fields = ();
	while(my @values = $sth->fetchrow_array()) {
		$fields{$values[0]} = $values[1];
	}
	$sth->finish;
	foreach my $uid (keys %fields) {
		if ($uid eq 'super_user') { $fields{$uid} = 'password' }
		my @saltchars = ('a'..'z','A'..'Z','0'..'9',',','/');
		srand(time() ^ ($$ + ($$ << 15)));
		my $salt = $saltchars[int(rand(64))];
		$salt .= $saltchars[int(rand(64))];
		my $newpw = crypt($fields{$uid}, $salt);
		my $sql = "update users set password = '$newpw' where user_acct = '$uid'";
		$sth = $dbh->prepare($sqlstmt);
		$sth->execute();
		$sth->finish;
	}
}

#
#############################################
# host_overrides changes
#

my $sqlstmt = 'describe host_overrides';
my $sth = $dbh->prepare($sqlstmt);
$sth->execute();
my %fields = ();
while(my @values = $sth->fetchrow_array()) {
	$fields{$values[0]} = 1;
}
$sth->finish;


#
#  add check_period 
#

unless (defined $fields{'check_period'}) {
	print "\n\tUpdating table host_overrides...\n";
	$dbh->do('alter table host_overrides add check_period SMALLINT(4) UNSIGNED after host_id');
}

#
#  drop status 
#

if (defined $fields{'status'}) {
	$dbh->do("ALTER TABLE host_overrides drop column status");
}


$sqlstmt = 'describe service_overrides';
$sth = $dbh->prepare($sqlstmt);
$sth->execute();
%fields = ();
while(my @values = $sth->fetchrow_array()) {
	$fields{$values[0]} = 1;
}
$sth->finish;


#
#  drop check_command 
#

if (defined $fields{'check_command'}) {
	print "\n\tUpdating table service_overrides...\n";
	$dbh->do("ALTER TABLE service_overrides drop column check_command");
}

#
#  drop status 
#

if (defined $fields{'status'}) {
	$dbh->do("ALTER TABLE service_overrides drop column status");
}

#
# Add service groups to asset list
#


$sqlstmt = "select * from access_list where object = 'servicegroups'";
$sth = $dbh->prepare($sqlstmt);
$sth->execute();
my $check = undef;
while(my @values = $sth->fetchrow_array()) {
	$check = 1;
}
$sth->finish;

unless ($check) {
	$dbh->do("insert into access_list values('servicegroups','design_manage','1','add,modify,delete')");

}

#
# Resource macro 
#
$dbh->do("update access_list set object = 'nagios_resource_macros' where object like '%resource%' and type = 'control'");


print "\n\tDistributing files...\n";

opendir(DIR, './images') || print "\n\n./images $!";
while (my $file = readdir(DIR)) {
	if ($file =~/^\./) { next }
	print "\n\t$doc_root/monarch/images/$file";
	copy("./images/$file","$doc_root/monarch/images/$file") || print "\n\nerror: $!";
}
close(DIR);
opendir(DIR, './doc') || print "\n\n./doc $!";
while (my $file = readdir(DIR)) {
	if ($file =~/^\./) { next }
	print "\n\t$doc_root/monarch/doc/$file";
	copy("./doc/$file","$doc_root/monarch/doc/$file") || print "\n\nerror: $!";
}
close(DIR);

my @files = ('monarch.js','nicetitle.js','monarch.css','ua.js','ftiens4.js');
foreach my $file (@files) {
	copy("./$file","$doc_root/monarch/$file") || print "\n\n./$file $doc_root/monarch/$file $!";
	print "\n\t$doc_root/monarch/$file";
}

@files = ('MonarchFile.pm','MonarchForms.pm','MonarchStorProc.pm','MonarchExternals.pm','MonarchDoc.pm','MonarchLoad.pm');
foreach my $file (@files) {
	copy("./$file","$monarch_home/lib/$file") || print "\n\nnot copied ./$file $monarch_home/lib/$file $!";
	print "\n\t$monarch_home/lib/$file";
}
@files = ('nagios_reload');
foreach my $file (@files) {
	copy("./$file","$monarch_home/bin/$file") || print "\n\nnot copied ./$file $monarch_home/bin/$file $!\n";
	print "\n\t$monarch_home/bin/$file";
}

open(FILE, "< ./monarch.cgi") || print "\n\n./monarch.cgi $!";
my $out_to_file = undef;
while (my $line = <FILE>) {
	if ($line =~ /STRINGTOREPLACE/) {
		$line =~ s/STRINGTOREPLACE/$monarch_home/;
	}
	$out_to_file .= $line;
}
close(FILE);

print "\n\tWriting $cgi_bin/monarch.cgi";
open(FILE, "> $cgi_bin/monarch.cgi") || print "\n\nerror: $!";
print FILE $out_to_file;
close(FILE);

print "\n\n\tUpdate complete.\n\n";
